package com.efetiva.wiiser.server.business;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.Iterator;

import com.efetiva.wiiser.server.entity.Activity;
import com.efetiva.wiiser.server.entity.CategoryActivity;
import com.efetiva.wiiser.server.entity.ScheduleActivity;
import com.efetiva.wiiser.server.repository.ActivityRepository;
import com.efetiva.wiiser.server.repository.interfaceBusiness.IActivityControl;
import com.efetiva.wiiser.server.repository.interfaceRepository.IActivityRepository;
import com.efetiva.wiser.exception.ExistingObjectException;
import com.efetiva.wiser.exception.NonExistingObjectException;
import com.efetiva.wiser.exception.NullObjectException;

public class ActivityControl implements IActivityControl {

	private IActivityRepository activities;

	/**
	 * 
	 */
	public ActivityControl() {
		activities = new ActivityRepository();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see wiiser.server.business.IActivityControl#modify(com.efetiva.wiser.entity.Activity)
	 */
	public void modify(Activity activity, Collection categories,
			Collection schedules) throws NonExistingObjectException,
			ExistingObjectException, NullObjectException {		

		// Modifying categories
		CategoryActivityControl categoryActivityControl = new CategoryActivityControl();
		categoryActivityControl.removeByActivity(activity.getId());
		Iterator iCategories = categories.iterator();
		while (iCategories.hasNext()) {
			CategoryActivity categoryActivity = (CategoryActivity) iCategories
			.next();
			categoryActivityControl.register(categoryActivity);
		}

		// Modifying schedules
		ScheduleActivityControl scheduleActivityControl = new ScheduleActivityControl();
		scheduleActivityControl.removeByActivity(activity.getId());
		Iterator iSchedules = schedules.iterator();
		while (iSchedules.hasNext()) {
			ScheduleActivity scheduleActivity = (ScheduleActivity) iSchedules
			.next();
			scheduleActivityControl.register(scheduleActivity);
		}

		this.activities.modify(activity);			


	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see wiiser.server.business.IActivityControl#register(com.efetiva.wiser.entity.Activity)
	 */
	public void register(Activity activity, Collection categoryActiviy,
			Collection scheduleActivities) throws ExistingObjectException,
			NullObjectException {		
		this.activities.register(activity);

		// Add the relationship Activity - Category
		CategoryActivityControl categoryControl = new CategoryActivityControl();
		Iterator categoryIterator = categoryActiviy.iterator();
		while (categoryIterator.hasNext()) {
			CategoryActivity categoryActivity = (CategoryActivity) categoryIterator
			.next();
			categoryControl.register(categoryActivity);
		}

		// Add the relationship Activity - Schedule
		ScheduleActivityControl scheduleControl = new ScheduleActivityControl();
		Iterator scheduleIterator = scheduleActivities.iterator();
		while (scheduleIterator.hasNext()) {
			ScheduleActivity scheduleActivity = (ScheduleActivity) scheduleIterator
			.next();
			scheduleControl.register(scheduleActivity);
		}

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see wiiser.server.business.IActivityControl#remove(com.efetiva.wiser.entity.Activity)
	 */
	public void remove(Activity activity) throws NonExistingObjectException,
	NullObjectException {		

		// Find category-activity
		CategoryActivityControl categoryControl = new CategoryActivityControl();
		Collection categoryActivities = categoryControl
		.searchByActivity(activity.getId());
		Iterator iterator = categoryActivities.iterator();
		while (iterator.hasNext()) {
			CategoryActivity categoryActivity = (CategoryActivity) iterator
			.next();
			categoryControl.remove(categoryActivity);
		}

		/*// Find schedule-activity
		 * Comentado pois gerava nonUniqueObject - passado pra interfaceWeb
				ScheduleActivityControl scheduleControl = new ScheduleActivityControl();
				Collection scheduleActivities = scheduleControl
						.searchByActivity(activity.getId());
				Iterator iteratorActivity = scheduleActivities.iterator();
				while (iteratorActivity.hasNext()) {
					ScheduleActivity scheduleActivity = (ScheduleActivity) iteratorActivity
							.next();
					scheduleControl.remove(scheduleActivity);
				}*/

		/* Remove a atividade */
		this.activities.remove(activity);		

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see wiiser.server.business.IActivityControl#search(java.lang.Integer)
	 */
	public Activity search(Integer ID) throws NonExistingObjectException,
	NullObjectException {
		Activity activity = null;		
		ScheduleActivityControl control = new ScheduleActivityControl();
		activity = this.activities.search(ID);
		if (activity != null) {
			Collection collScheduleActivity = control.searchByActivity(ID);
			activity.setSchedules(collScheduleActivity);
		} else {
			throw new NonExistingObjectException("Activity");
		}		
		return activity;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see wiiser.server.business.IActivityControl#searchActivityByDate(java.util.Date)
	 */
	public Collection searchByDate(Date date, Integer languageID) throws NullObjectException{
		Collection collActivity = new ArrayList();		
		collActivity = this.activities.searchByDate(date, languageID);		
		return collActivity;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see wiiser.server.business.IActivityControl#searchActivityByArea(java.lang.Integer)
	 */
	public Collection searchByArea(Integer areaID, Integer languageID) throws NullObjectException{
		Collection activities = this.activities.searchByArea(areaID, languageID);		
		return activities;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see wiiser.server.business.IActivityControl#searchActivityByPlace(java.lang.Integer)
	 */
	public Collection searchByPlace(Integer placeID) throws NullObjectException{
		Collection activities = this.activities.searchByPlace(placeID);
		return activities;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see wiiser.server.business.IActivityControl#searchAllActivities()
	 */
	public Collection searchAll() {
		return this.activities.searchAll();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see wiiser.server.business.IActivityControl#searchByLanguage(java.lang.Integer)
	 */
	public Collection searchByLanguage(Integer languageID) throws NullObjectException{
		return this.activities.searchByLanguage(languageID);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see wiiser.server.business.IActivityControl#searchByName(java.lang.String)
	 */
	public Activity searchByName(String name, Integer languageID) throws NullObjectException{
		Activity activity = this.activities.searchByName(name, languageID);		
		return null;
	}

}
