package com.efetiva.wiiser.server.business;

import java.util.Collection;

import com.efetiva.wiiser.server.entity.Area;
import com.efetiva.wiiser.server.repository.AreaRepository;
import com.efetiva.wiiser.server.repository.interfaceBusiness.IAreaControl;
import com.efetiva.wiiser.server.repository.interfaceRepository.IAreaRepository;
import com.efetiva.wiser.exception.ExistingObjectException;
import com.efetiva.wiser.exception.NonExistingObjectException;
import com.efetiva.wiser.exception.NullObjectException;

public class AreaControl implements IAreaControl {

	private IAreaRepository areas;

	/***************************************************************************
	 * Constructor of AreaControl
	 * 
	 */
	public AreaControl() {
		this.areas = new AreaRepository();
	}

	/***************************************************************************
	 * Stores an Area object in the DB.
	 * 
	 * @param area
	 *            area to be stored
	 * @throws ExistingObjectException
	 * @throws NullObjectException
	 */
	public void register(Area area) throws ExistingObjectException,
	NullObjectException {		
		this.areas.register(area);		

	}

	/***************************************************************************
	 * Returns an Area object searched by its identification in the DB.
	 * <p>
	 * If the Area isn't found it returns null.
	 * 
	 * @param ID
	 *            the identification in the DB
	 * @return an Area object
	 * @throws NullObjectException
	 */
	public Area search(Integer id) throws NullObjectException {
		Area area = null;	
		area = this.areas.search(id);
		return area;
	}

	/***************************************************************************
	 * Removes an Area object from the DB
	 * 
	 * @param area
	 *            area to be removed
	 * @throws NonExistingObjectException
	 * @throws NullObjectException
	 */
	public void remove(Area area) throws NonExistingObjectException,
	NullObjectException {		
		this.areas.remove(area);
	}

	/***************************************************************************
	 * Updates an Area object in the DB
	 * 
	 * @param area
	 *            area to be updated
	 * @throws NonExistingObjectException
	 * @throws NullObjectException
	 */
	public void modify(Area area) throws NonExistingObjectException,
	NullObjectException {		
		this.areas.modify(area);		
	}

	/***************************************************************************
	 * Returns a collection of all Area object stored in the DB.
	 * <p>
	 * If any Area isn't found it returns an empty collection.
	 * 
	 * @return an Area collection
	 */
	public Collection searchAll() {
		return this.areas.searchAll();
	}

	/***************************************************************************
	 * Returns a collection of all Area object stored in the DB. with the
	 * specific language
	 * <p>
	 * If any Area isn't found it returns an empty collection.
	 * 
	 * @return an Area collection
	 */
	public Collection searchByLanguage(Integer languageID) throws NullObjectException{
		return this.areas.searchByLanguage(languageID);
	}

	public Area searchByName(String name) throws NullObjectException {
		Area area = this.areas.searchByName(name);		
		return area;
	}

}
