package com.efetiva.wiiser.server.business;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import com.efetiva.wiiser.server.entity.DynamicInfo;
import com.efetiva.wiiser.server.entity.ScheduleInfo;
import com.efetiva.wiiser.server.repository.DynamicInfoRepository;
import com.efetiva.wiiser.server.repository.interfaceBusiness.IDynamicInfoControl;
import com.efetiva.wiiser.server.repository.interfaceRepository.IDynamicInfoRepository;
import com.efetiva.wiser.exception.ExistingObjectException;
import com.efetiva.wiser.exception.NonExistingObjectException;
import com.efetiva.wiser.exception.NullObjectException;

public class DynamicInfoControl implements IDynamicInfoControl {

	private IDynamicInfoRepository dynamicInfos;

	/***************************************************************************
	 * Constructor of DynamicInfoControl
	 * 
	 */
	public DynamicInfoControl() {
		this.dynamicInfos = new DynamicInfoRepository();
	}

	/***************************************************************************
	 * Stores a DynamicInfo object in the DB
	 * 
	 * @param dynamicInfo
	 *            DynamicInfo to be stored
	 * @throws ExistingObjectException
	 * @throws NullObjectException
	 */
	public void register(DynamicInfo dynamicInfo, Collection scheduleInfos)
	throws ExistingObjectException, NullObjectException {		
		this.dynamicInfos.register(dynamicInfo);
		// Add the relationship schedule - dynamicInfo
		if (scheduleInfos != null && !scheduleInfos.isEmpty()) {
			ScheduleInfoControl control = new ScheduleInfoControl();
			Iterator iterator = scheduleInfos.iterator();
			while (iterator.hasNext()) {
				ScheduleInfo scheduleInfo = (ScheduleInfo) iterator.next();
				control.register(scheduleInfo);
			}
		}		
	}

	/***************************************************************************
	 * Returns a DynamicInfo object searched by its identification in the DB.
	 * <p>
	 * If the DynamicInfo isn't found it returns null.
	 * 
	 * @param ID
	 *            the identification in the DB
	 * @return a DynamicInfo object
	 * @throws NullObjectException
	 */
	public DynamicInfo search(Integer ID) throws NullObjectException {
		DynamicInfo dynamicInfo = this.dynamicInfos.search(ID);		
		return dynamicInfo;
	}

	/***************************************************************************
	 * Removes a DynamicInfo object from the DB
	 * 
	 * @param info
	 *            DynamicInfo to be removed
	 * @throws NonExistingObjectException
	 * @throws NullObjectException
	 */
	public void remove(DynamicInfo info) throws NonExistingObjectException,
	NullObjectException {
		if (info.getId() != null) {

			/* Remove DynamicInfo */
			this.dynamicInfos.remove(info);

			// Find schedule - dynamicInfo 
			//Comentado pois tava dando nonUniqueObject, foi copiado na interface Web
			/*ScheduleInfoControl control = new ScheduleInfoControl();
			Collection scheduleInfos = control.searchByDynamicInfo(info.getId());
			Iterator iterator = scheduleInfos.iterator();
			while (iterator.hasNext()) {
				ScheduleInfo scheduleInfo = (ScheduleInfo) iterator.next();
				control.remove(scheduleInfo);
			}*/			

		} else {
			throw new NonExistingObjectException("DynamicInfo");// TODO
			// InfoInexistenteException
		}

	}

	/***************************************************************************
	 * Updates a DynamicInfo object in the DB
	 * 
	 * @param info
	 *            DynamicInfo to be updated
	 * @throws NonExistingObjectException
	 * @throws NullObjectException
	 */
	public void modify(DynamicInfo info, Collection schedules)
	throws NonExistingObjectException, NullObjectException {
		if (info.getId() != null) {
			this.dynamicInfos.modify(info);
		} else {
			throw new NonExistingObjectException("DynamicInfo");// TODO
			// InexistingDynamicInfoException
		}

	}

	/***************************************************************************
	 * Returns a collection of all DynamicInfo object stored in the DB.
	 * <p>
	 * If any DynamicInfo isn't found it returns an empty collection.
	 * 
	 * @return a DynamicInfo collection
	 */
	public Collection searchAll() {
		return this.dynamicInfos.searchAll();
	}

	/***************************************************************************
	 * Returns a collection of all DynamicInfo object stored in the DB with the
	 * specific languageID.
	 * <p>
	 * If any DynamicInfo isn't found it returns an empty collection.
	 * 
	 * @return a DynamicInfo collection
	 * @throws NullObjectException
	 */
	public Collection searchByLanguage(Integer languageID)
	throws NullObjectException {
		Collection dynamicInfo = new ArrayList();		
		dynamicInfo = this.dynamicInfos.searchByLanguage(languageID);		
		return dynamicInfo;
	}

	public DynamicInfo selectMaxId(){
		return this.dynamicInfos.selectMaxId();
	}
}
