package com.efetiva.wiiser.server.business;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import com.efetiva.wiiser.server.entity.CategoryPlace;
import com.efetiva.wiiser.server.entity.Place;
import com.efetiva.wiiser.server.entity.PlaceArea;
import com.efetiva.wiiser.server.repository.PlaceRepository;
import com.efetiva.wiiser.server.repository.interfaceBusiness.IPlaceControl;
import com.efetiva.wiiser.server.repository.interfaceRepository.IPlaceRepository;
import com.efetiva.wiser.exception.ExistingObjectException;
import com.efetiva.wiser.exception.NonExistingObjectException;
import com.efetiva.wiser.exception.NullObjectException;

public class PlaceControl implements IPlaceControl {

	private IPlaceRepository places;

	/***************************************************************************
	 * Constructor of PlaceControl
	 * 
	 */
	public PlaceControl() {
		this.places = new PlaceRepository();
	}

	/***************************************************************************
	 * Stores a Place object in the DB
	 * 
	 * @param place
	 *            place to be stored
	 * @throws ExistingObjectException
	 * @throws NullObjectException
	 * @throws NonExistingObjectException
	 */
	public void register(Place place, Collection collCategoryPlace,
			Collection collPlaceArea) throws ExistingObjectException,
			NullObjectException, NonExistingObjectException {		
		this.places.register(place);

		// Add the relationship Place - Category
		CategoryPlaceControl categoryControl = new CategoryPlaceControl();
		Iterator iterator = collCategoryPlace.iterator();
		while (iterator.hasNext()) {
			CategoryPlace categoryPlace = (CategoryPlace) iterator.next();
			categoryControl.register(categoryPlace);
		}

		// Add the relationship Place - Category
		PlaceAreaControl placeAreaControl = new PlaceAreaControl();
		Iterator iteratorArea = collPlaceArea.iterator();
		while (iteratorArea.hasNext()) {
			PlaceArea placeArea = (PlaceArea) iteratorArea.next();
			placeAreaControl.register(placeArea);
		}		
	}

	/***************************************************************************
	 * Returns a Place object searched by its identification in the DB.
	 * <p>
	 * If the Place isn't found it returns null.
	 * 
	 * @param ID
	 *            the identification in the DB
	 * @return a Place object
	 * @throws NullObjectException
	 */
	public Place search(Integer placeID) throws NullObjectException {
		Place place = this.places.search(placeID);		
		return place;
	}

	/***************************************************************************
	 * Returns a Place object searched by its name in the DB.
	 * <p>
	 * If the Place isn't found it returns null.
	 * 
	 * @param ID
	 *            the identification in the DB
	 * @return a Place object
	 * @throws NullObjectException
	 */
	public Place searchByName(String placeName, Integer languageID) throws NullObjectException {
		Place place = this.places.searchByName(placeName, languageID);		
		return place;
	}

	/***************************************************************************
	 * Removes a Place object from the DB
	 * 
	 * @param place
	 *            place to be removed
	 * @throws NonExistingObjectException
	 * @throws NullObjectException
	 */
	public void remove(Place place) throws NonExistingObjectException,
	NullObjectException {		
		this.places.remove(place);

		// Find category-place
		CategoryPlaceControl control = new CategoryPlaceControl();
		Collection collCategories = control.searchByPlace(place.getId());
		Iterator iterator = collCategories.iterator();
		while (iterator.hasNext()) {
			CategoryPlace categoryPlace = (CategoryPlace) iterator.next();
			control.remove(categoryPlace);
		}

		// Find place - area
		PlaceAreaControl placeAreaControl = new PlaceAreaControl();
		Collection collPlaceAreas = placeAreaControl.searchByPlace(place
				.getId());
		Iterator iteratorAreas = collPlaceAreas.iterator();
		while (iteratorAreas.hasNext()) {
			PlaceArea placeArea = (PlaceArea) iteratorAreas.next();
			placeAreaControl.remove(placeArea);
		}		
	}

	/***************************************************************************
	 * Updates a Place object in the DB
	 * 
	 * @param place
	 *            place to be updated
	 * @throws NonExistingObjectException
	 * @throws NullObjectException
	 * @throws ExistingObjectException
	 */
	public void modify(Place place, Collection categories,
			Collection collPlacesAreas) throws NonExistingObjectException,
			NullObjectException, ExistingObjectException {
		if (place.getId() != null) {
			this.places.modify(place);
			// modifying categories
			CategoryPlaceControl categoryPlaceControl = new CategoryPlaceControl();
			categoryPlaceControl.removeByPlace(place.getId());
			Iterator iCategories = categories.iterator();
			while (iCategories.hasNext()) {
				CategoryPlace categoryPlace = (CategoryPlace) iCategories
				.next();
				categoryPlaceControl.register(categoryPlace);
			}

			// Modifying place - area
			PlaceAreaControl placeAreaControl = new PlaceAreaControl();
			placeAreaControl.removeByPlace(place.getId());
			Iterator iteratorAreas = collPlacesAreas.iterator();
			while (iteratorAreas.hasNext()) {
				PlaceArea placeArea = (PlaceArea) iteratorAreas.next();
				placeAreaControl.register(placeArea);
			}

		} else {
			throw new NonExistingObjectException("Place");
		}
	}

	/***************************************************************************
	 * Returns a collection of all Place object stored in the DB.
	 * <p>
	 * If any Place isn't found it returns an empty collection.
	 * 
	 * @return a Place collection
	 */
	public Collection searchAll() {
		return this.places.searchAll();
	}

	/***************************************************************************
	 * Returns a collection of all Place object stored in the DB with the
	 * specific language.
	 * <p>
	 * If any Place isn't found it returns an empty collection.
	 * 
	 * @return a Place collection
	 * @throws NullObjectException
	 */
	public Collection searchByLanguage(Integer languageID)
	throws NullObjectException {
		Collection places = new ArrayList();		
		places = this.places.searchByLanguage(languageID);		
		return places;
	}

}
