package com.efetiva.wiiser.server.business;

import java.util.ArrayList;
import java.util.Collection;

import com.efetiva.wiiser.server.entity.ScheduleActivity;
import com.efetiva.wiiser.server.repository.ScheduleActivityRepository;
import com.efetiva.wiiser.server.repository.interfaceBusiness.IScheduleActivityControl;
import com.efetiva.wiiser.server.repository.interfaceRepository.IScheduleActivityRepository;
import com.efetiva.wiser.exception.NonExistingObjectException;
import com.efetiva.wiser.exception.NullObjectException;

public class ScheduleActivityControl implements IScheduleActivityControl {

	private IScheduleActivityRepository scheduleActivities;

	/***************************************************************************
	 * Constructor of ScheduleActivityControl
	 * 
	 */
	public ScheduleActivityControl() {
		this.scheduleActivities = new ScheduleActivityRepository();
	}

	/***************************************************************************
	 * Stores a ScheduleActivity object in the DB
	 * 
	 * @param scheduleActivity
	 *            ScheduleActivity to be stored
	 * @throws NullObjectException
	 */
	public void register(ScheduleActivity scheduleActivity)
			throws NullObjectException {		
			this.scheduleActivities.register(scheduleActivity);		
	}

	/***************************************************************************
	 * Removes a ScheduleActivity object from the DB
	 * 
	 * @param schedule
	 *            ScheduleActivity to be removed
	 * @throws NullObjectException
	 */
	public void remove(ScheduleActivity schedule) throws NullObjectException {		
			this.scheduleActivities.remove(schedule);		
	}

	/***************************************************************************
	 * Updates a ScheduleActivity object in the DB
	 * 
	 * @param schedule
	 *            ScheduleActivity to be updated
	 * @throws NonExistingObjectException
	 */
	public void modify(ScheduleActivity schedule)
			throws NonExistingObjectException,NullObjectException {		
			this.scheduleActivities.modify(schedule);		

	}

	/**
	 * Searches all existing ScheduleActivity
	 * 
	 * @return a ScheduleActivity object collection
	 */
	public Collection searchAll() {
		return this.scheduleActivities.searchAll();
	}

	/***************************************************************************
	 * Returns a collection of ScheduleActivity object searched by activity
	 * identification in the DB.
	 * <p>
	 * If any ScheduleActivity isn't found it returns an empty collection.
	 * 
	 * @param ID
	 *            activity identification in the DB
	 * @return a ScheduleActivity object
	 */
	public Collection searchByActivity(Integer ID) throws NullObjectException{
		Collection schedules = new ArrayList();		
			schedules = this.scheduleActivities.searchByActivity(ID);		
		return schedules;
	}

	/***************************************************************************
	 * Returns a collection of ScheduleActivity object searched by activity name
	 * in the DB.
	 * <p>
	 * If any ScheduleActivity isn't found it returns an empty collection.
	 * 
	 * @param activityName
	 *            activity name in the DB
	 * @return a ScheduleActivity object
	 */
	public Collection searchByActivityName(String activityName, Integer language) throws NullObjectException{
		Collection schedules = new ArrayList();		
			schedules = this.scheduleActivities.searchByActivityName(
					activityName, language);		
		return schedules;
	}

	public void removeByActivity(Integer activityID) throws NullObjectException{
		this.scheduleActivities.removeByActivity(activityID);
	}

	public ScheduleActivity search(Integer scheduleID) throws NullObjectException{
		return this.scheduleActivities.search(scheduleID);
	}

}
