package com.efetiva.wiiser.server.business;

import java.util.Collection;
import java.util.Date;

import com.efetiva.wiiser.server.entity.ScheduleInfo;
import com.efetiva.wiiser.server.repository.ScheduleInfoRepository;
import com.efetiva.wiiser.server.repository.interfaceBusiness.IScheduleInfoControl;
import com.efetiva.wiiser.server.repository.interfaceRepository.IScheduleInfoRepository;
import com.efetiva.wiser.exception.NullObjectException;

public class ScheduleInfoControl implements IScheduleInfoControl {

	private IScheduleInfoRepository scheduleInfos;

	/***************************************************************************
	 * Constructor of ScheduleInfoControl
	 * 
	 */
	public ScheduleInfoControl() {
		this.scheduleInfos = new ScheduleInfoRepository();
	}

	/***************************************************************************
	 * Stores a ScheduleInfo object in the DB
	 * 
	 * @param scheduleInfo
	 *            ScheduleInfo to be stored
	 * @throws NullObjectException
	 */
	public void register(ScheduleInfo scheduleInfo) throws NullObjectException{	
			this.scheduleInfos.register(scheduleInfo);		
	}

	/***************************************************************************
	 * Removes a ScheduleInfo object from the DB
	 * 
	 * @param schedule
	 *            ScheduleInfo to be removed
	 * @throws NullObjectException
	 */
	public void remove(ScheduleInfo schedule) throws NullObjectException{		
			this.scheduleInfos.remove(schedule);		
	}

	/***************************************************************************
	 * Updates a ScheduleInfo object in the DB
	 * 
	 * @param schedule
	 *            ScheduleInfo to be updated
	 */
	public void modify(ScheduleInfo schedule) throws NullObjectException{		
			this.scheduleInfos.modify(schedule);		
	}

	/**
	 * Searches all existing ScheduleInfo
	 * 
	 * @return a ScheduleInfo object collection
	 */
	public Collection searchAll() {
		return this.scheduleInfos.searchAll();
	}

	/**
	 * Searches all existing ScheduleInfo with specific DynamicInfo id
	 * 
	 * @return a ScheduleInfo object collection
	 */
	public Collection searchByDynamicInfo(Integer id) throws NullObjectException{
		return this.scheduleInfos.searchByDynamicInfo(id);
	}

	public void removeByDynamicInfo(Integer infoID) throws NullObjectException{
		this.scheduleInfos.removeByDynamicInfo(infoID);
	}

	public Collection searchByHour(Date date) throws NullObjectException{
		return this.scheduleInfos.searchByHour(date);
	}

}
