package com.efetiva.wiiser.server.business;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.Iterator;

import com.efetiva.wiiser.server.entity.CategoryService;
import com.efetiva.wiiser.server.entity.ScheduleService;
import com.efetiva.wiiser.server.entity.Service;
import com.efetiva.wiiser.server.repository.ServiceRepository;
import com.efetiva.wiiser.server.repository.interfaceBusiness.IServiceControl;
import com.efetiva.wiiser.server.repository.interfaceRepository.IServiceRepository;
import com.efetiva.wiser.exception.ExistingObjectException;
import com.efetiva.wiser.exception.NonExistingObjectException;
import com.efetiva.wiser.exception.NullObjectException;

public class ServiceControl implements IServiceControl {

	private IServiceRepository services;

	/***************************************************************************
	 * Constructor of ServiceControl
	 * 
	 */
	public ServiceControl() {
		services = new ServiceRepository();
	}

	/***************************************************************************
	 * Stores a Service object in the DB
	 * 
	 * @param service
	 *            service to be stored
	 * @throws ExistingObjectException
	 * @throws NullObjectException
	 */
	public void register(Service service, Collection collCategoryService,
			Collection scheduleServices) throws ExistingObjectException,
			NullObjectException {		
		this.services.register(service);

		// Add the relationship Service - Category
		CategoryServiceControl categoryControl = new CategoryServiceControl();
		Iterator categoryIterator = collCategoryService.iterator();
		while (categoryIterator.hasNext()) {
			CategoryService categoryService = (CategoryService) categoryIterator
			.next();
			categoryControl.register(categoryService);
		}

		// Add the relationship Service - Schedule
		ScheduleServiceControl scheduleControl = new ScheduleServiceControl();
		Iterator scheduleIterator = scheduleServices.iterator();
		while (scheduleIterator.hasNext()) {
			ScheduleService scheduleService = (ScheduleService) scheduleIterator
			.next();
			scheduleControl.register(scheduleService);
		}		
	}

	/***************************************************************************
	 * Returns a Service object searched by its identification in the DB.
	 * <p>
	 * If the Service isn't found it returns null.
	 * 
	 * @param ID
	 *            the identification in the DB
	 * @return a Service object
	 * @throws NullObjectException
	 */
	public Service search(Integer ID) throws NullObjectException {
		Service service = null;		
		ScheduleServiceControl control = new ScheduleServiceControl();
		Collection collScheduleService = control.searchByService(ID);
		service = this.services.search(ID);
		if (collScheduleService != null && !collScheduleService.isEmpty()) {
			service.setSchedules(collScheduleService);
		}		
		return service;

	}

	/***************************************************************************
	 * Returns a Service object searched by specific date.
	 * <p>
	 * If any Service isn't found it returns an empty collection.
	 * 
	 * @param date
	 *            the specific date
	 * @return a Service object collection
	 * @throws NullObjectException
	 */
	public Collection searchServiceByDate(Date date) throws NullObjectException {
		Collection collService = new ArrayList();

		collService = this.services.searchServiceByDate(date);
		Iterator it = collService.iterator();
		while (it.hasNext()) {
			Service service = (Service) it.next();
			ScheduleServiceControl control = new ScheduleServiceControl();
			Collection collScheduleService = control
			.searchByService(service.getId());
			service.setSchedules(collScheduleService);
		}		
		return collService;
	}

	/***************************************************************************
	 * Removes a Service object from the DB
	 * 
	 * @param service
	 *            service to be removed
	 * @throws NonExistingObjectException
	 * @throws NullObjectException
	 */
	public void remove(Service service) throws NonExistingObjectException,
	NullObjectException {		

		// Find category-service
		CategoryServiceControl categoryControl = new CategoryServiceControl();
		Collection categoryServices = categoryControl
		.searchByService(service.getId());
		Iterator iterator = categoryServices.iterator();
		while (iterator.hasNext()) {
			CategoryService categoryService = (CategoryService) iterator
			.next();
			categoryControl.remove(categoryService);
		}

		/*// Find schedule-service
		 * comentado pois gerava nonUniqueObject - levado pra interfaceWeb
			ScheduleServiceControl scheduleControl = new ScheduleServiceControl();
			Collection scheduleServices = scheduleControl
					.searchByService(service.getId());
			Iterator iteratorService = scheduleServices.iterator();
			while (iteratorService.hasNext()) {
				ScheduleService scheduleService = (ScheduleService) iteratorService
						.next();
				scheduleControl.remove(scheduleService);
			}*/

		this.services.remove(service);		
	}

	/***************************************************************************
	 * Updates a Service object in the DB
	 * 
	 * @param service
	 *            service to be updated
	 * @throws NonExistingObjectException
	 * @throws NullObjectException
	 * @throws ExistingObjectException
	 */
	public void modify(Service service, Collection categories,
			Collection schedules) throws NonExistingObjectException,
			NullObjectException, ExistingObjectException {		
		/* Evita null pointer */
		if (categories == null)
			categories = new ArrayList();
		if (schedules == null)
			schedules = new ArrayList();

		this.services.modify(service);

		// Modifying categories
		CategoryServiceControl categoryServiceControl = new CategoryServiceControl();
		categoryServiceControl.removeByService(service.getId());
		Iterator iCategories = categories.iterator();
		while (iCategories.hasNext()) {
			CategoryService categoryService = (CategoryService) iCategories
			.next();
			categoryServiceControl.register(categoryService);
		}

		// Modifying schedules
		ScheduleServiceControl scheduleServiceControl = new ScheduleServiceControl();
		scheduleServiceControl.removeByService(service.getId());
		Iterator iSchedules = schedules.iterator();
		while (iSchedules.hasNext()) {
			ScheduleService scheduleService = (ScheduleService) iSchedules
			.next();
			scheduleServiceControl.register(scheduleService);
		}
	}

	/**
	 * Searches all existing services
	 * 
	 * @return a Service object collection
	 */
	public Collection searchAllServices() {

		return this.services.searchAllServices();
	}

	/**
	 * Searches all existing services
	 * 
	 * @return a Service object collection
	 */
	public Collection searchServicesByLanguage(Integer languageID) throws NullObjectException{
		Collection services = new ArrayList();	
		services = this.services.searchServicesByLanguage(languageID);
		return services;
	}

	/**
	 * 
	 * @param placeID
	 * @return
	 */
	public Collection searchServiceByPlace(Integer placeID) throws NullObjectException{
		Collection services = null;		
		services = this.services.searchServiceByPlace(placeID);		
		return services;
	}

	public Collection searchServiceByArea(Integer rgb, Integer languageID) throws NullObjectException{
		Collection services = null;		
		services = this.services.searchServiceByArea(rgb, languageID);		
		return services;
	}

	public Service searchByName(String name, Integer languageID) throws NullObjectException{
		Service service = null;		
		service = this.services.searchByName(name, languageID);		
		return service;
	}
}