package com.efetiva.wiiser.server.business;

import java.util.Collection;

import com.efetiva.wiiser.server.entity.User;
import com.efetiva.wiiser.server.repository.UserRepository;
import com.efetiva.wiiser.server.repository.interfaceBusiness.IUserControl;
import com.efetiva.wiiser.server.repository.interfaceRepository.IUserRepository;
import com.efetiva.wiser.exception.NonExistingObjectException;
import com.efetiva.wiser.exception.NullObjectException;

public class UserControl implements IUserControl {

	private IUserRepository users;

	/***************************************************************************
	 * Constructor of UserControl
	 * 
	 */
	public UserControl() {
		this.users = new UserRepository();
	}

	/***************************************************************************
	 * Stores a User object in the DB
	 * 
	 * @param user
	 *            user to be stored
	 * @throws NullObjectException
	 */
	public void register(User user) throws NullObjectException {		
		this.users.register(user);		
	}

	/***************************************************************************
	 * Returns a User object searched by its username (identification in the
	 * DB).
	 * <p>
	 * If the User isn't found it returns null.
	 * 
	 * @param username
	 *            username that is the identification in the DB
	 * @return a User object
	 */
	public User search(String username) throws NullObjectException{
		User user = this.users.search(username);		
		return user;
	}

	/***************************************************************************
	 * Removes a User object from the DB
	 * 
	 * @param user
	 *            user to be removed
	 * @throws NullObjectException
	 */
	public void remove(User user) throws NullObjectException {		
		this.users.remove(user);		
	}

	/***************************************************************************
	 * Updates a User object in the DB
	 * 
	 * @param user
	 *            user to be updated
	 * @throws NonExistingObjectException
	 */
	public void modify(User user) throws NonExistingObjectException, NullObjectException {		
		this.users.modify(user);		
	}

	public Collection searchAll() {
		return this.users.searchAll();
	}

}
