package com.efetiva.wiiser.server.comm;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.Iterator;

import javax.imageio.stream.FileImageInputStream;

import com.efetiva.wiiser.server.entity.Activity;
import com.efetiva.wiiser.server.entity.Area;
import com.efetiva.wiiser.server.entity.CategoryActivity;
import com.efetiva.wiiser.server.entity.CategoryPlace;
import com.efetiva.wiiser.server.entity.CategoryService;
import com.efetiva.wiiser.server.entity.Language;
import com.efetiva.wiiser.server.entity.Map;
import com.efetiva.wiiser.server.entity.Place;
import com.efetiva.wiiser.server.entity.PlaceArea;
import com.efetiva.wiiser.server.entity.ScheduleActivity;
import com.efetiva.wiiser.server.entity.ScheduleService;
import com.efetiva.wiiser.server.entity.Service;
import com.efetiva.wiiser.server.singleton.Facade;
import com.efetiva.wiser.entity.model.ActivityModel;
import com.efetiva.wiser.entity.model.AreaModel;
import com.efetiva.wiser.entity.model.CoordenadaModel;
import com.efetiva.wiser.entity.model.HeaderModel;
import com.efetiva.wiser.entity.model.LanguageModel;
import com.efetiva.wiser.entity.model.MapModel;
import com.efetiva.wiser.entity.model.PlaceModel;
import com.efetiva.wiser.entity.model.ScheduleModel;
import com.efetiva.wiser.entity.model.ServiceModel;
import com.efetiva.wiser.entity.util.Constants;
import com.efetiva.wiser.exception.NonExistingObjectException;
import com.efetiva.wiser.exception.NullObjectException;
import com.efetiva.wiser.interfaces.ServerFacade;
import communication.server.Server;

public class ServerCommunication implements ServerFacade {

	private Facade facade;

	private Server server;

	/* private ServerThreadComm threadInfo; */

	private static ServerCommunication SINGLE_INSTANCE = null;

	/**
	 * Constructor of ServerCommunication
	 * 
	 * @throws IOException
	 */
	private ServerCommunication() throws IOException {
		facade = Facade.getInstance();
		this.server = new Server(this/*, new File("./ContextAwareConfig.xml")*/);
		this.server.start();
		new ServerThreadComm(this.server);
	}

	public static ServerCommunication getInstance() throws IOException {
		if (SINGLE_INSTANCE == null) {
			SINGLE_INSTANCE = new ServerCommunication();
		}
		return SINGLE_INSTANCE;
	}

	/**
	 * Gets image Path, then returns a byte array of the image
	 * 
	 * @param Imagepath -
	 *            a string that identifies the image archive path
	 * @return a byte array of the image
	 */
	private byte[] getByteArray(String imagePath) {
		System.out.println("File to ByteArray request...");
		FileImageInputStream in = null;
		byte[] photo = null;
		try {
			in = new FileImageInputStream(new File(imagePath));
			photo = new byte[(int) in.length()];
			in.read(photo);
			System.out.println("File " + imagePath + " sent!!!");
		} catch (FileNotFoundException e) {
			System.out.println("File " + imagePath + " not found!!!");
		} catch (IOException e) {
			System.out.println("IO Exception!!!");
		}
		return photo;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see wiiser.server.comm.ServerFacade#requestActivityInformation(java.lang.Integer)
	 */
	public ActivityModel requestActivityInformation(Integer activityID) {

		System.out.println("Informao da Atividade...");

		ActivityModel activityModel = null;
		Collection categories = null;
		Iterator iCategory = null;

		try {
			Activity activity = facade.searchActivity(activityID);
			
			if (activity != null) {
				Collection categoryActivity = facade.searchCategoryActivityByActivity(activityID);
				
				// Categories
				iCategory = categoryActivity.iterator();
				categories = new ArrayList();
				while (iCategory.hasNext()) {
					CategoryActivity categoryA = (CategoryActivity) iCategory.next();
					categories.add(categoryA.getCategory().getName());
				}
				HeaderModel headerModel = new HeaderModel(activity.getId(),	activity.getName(), categories);
				
				// Collection scheduleModel
				Iterator iterator = activity.getSchedules().iterator();
				Collection schedules = new ArrayList();
				ScheduleActivity scheduleA = null;
				ScheduleModel scheduleM = null;
				while (iterator.hasNext()) {
					scheduleA = (ScheduleActivity) iterator.next();
					scheduleM = new ScheduleModel(Constants	.dayOfWeekToString(scheduleA.getDayOfWeek()),
								                  scheduleA.getHour(), null);
					schedules.add(scheduleM);
				}
				
				activityModel = new ActivityModel(headerModel, activity.getDescription(), 
												  getByteArray(activity.getPhoto()),
						                          schedules, activity.getValue(), null /* xml */);
				
			}
			
		} catch (NonExistingObjectException e) {
			e.getMessage();
		} catch (NullObjectException e) {
			e.getMessage();
		}

		System.out.println("Informao da Atividade retornado!!!");

		return activityModel;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see wiiser.server.comm.ServerFacade#requestDayActivities(java.lang.Integer)
	 */
	public Collection requestDayActivities(Date date, Integer languageID) {
		System.out.println("Atividades do dia...");

		Collection collHeaderModel = new ArrayList();
		Activity activity = null;
		Collection collCategoryActivity = null;
		Iterator iCategory = null;
		Collection categories = null;
		CategoryActivity categoryA = null;

		Collection collActivity = facade.searchActivityByDate(date,	languageID);
		Iterator iterator = collActivity.iterator();
		while (iterator.hasNext()) {
			activity = (Activity) iterator.next();
			collCategoryActivity = facade.searchCategoryActivityByActivity(activity.getId());

			// Categories
			iCategory = collCategoryActivity.iterator();
			categories = new ArrayList();
			while (iCategory.hasNext()) {
				categoryA = (CategoryActivity) iCategory.next();
				categories.add(categoryA.getCategory().getName());
			}
			HeaderModel headerModel = new HeaderModel(activity.getId(),
													  activity.getName(), categories);

			collHeaderModel.add(headerModel);
		}

		System.out.println("Atividades do dia enviada!!!");
		return collHeaderModel;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see wiiser.server.comm.ServerFacade#requestAreaActivities(java.lang.Integer,
	 *      java.lang.Integer)
	 */
	public Collection requestAreaActivities(Integer rgb, Integer languageID) {
		System.out.println("Atividades pela area...");

		Collection collHeaderModel = new ArrayList();
		Collection collCategoryActivity = null;
		Iterator iCategory = null;
		Activity activity = null;
		Collection categories = null;

		Collection collActivity = facade.searchActivityByArea(rgb, languageID);
		Iterator iterator = collActivity.iterator();
		while (iterator.hasNext()) {
			activity = (Activity) iterator.next();
			collCategoryActivity = facade.searchCategoryActivityByActivity(activity.getId());

			// Categories
			iCategory = collCategoryActivity.iterator();
			categories = new ArrayList();
			while (iCategory.hasNext()) {
				CategoryActivity categoryA = (CategoryActivity) iCategory.next();
				categories.add(categoryA.getCategory().getName());
			}

			HeaderModel headerModel = new HeaderModel(activity.getId(), activity.getName(), categories);
			collHeaderModel.add(headerModel);
		}
		System.out.println("Atividades pela area enviada!!!");
		return collHeaderModel;
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see wiiser.server.comm.ServerFacade#requestAllActivities()
	 */
	public Collection requestAllActivities(Integer languageID) {
		System.out.println("Procurar todos os servios...");

		Collection allActivities = null;
		Activity activity = null;
		Collection allActivitiesHeader = new ArrayList();
		HeaderModel headerM = null;
		Collection categories = null;
		Iterator iCategories = null;
		CategoryActivity categoryActivity = null;

		allActivities = facade.searchActivitiesByLanguage(languageID);

		Iterator itServices = allActivities.iterator();
		while (itServices.hasNext()) {
			activity = (Activity) itServices.next();
			Collection categoryCollection = facade.searchCategoryActivityByActivity(activity.getId());

			// Categories
			categories = new ArrayList();
			iCategories = categoryCollection.iterator();

			while (iCategories.hasNext()) {
				categoryActivity = (CategoryActivity) iCategories.next();
				categories.add(categoryActivity.getCategory().getName());
			}
			headerM = new HeaderModel(activity.getId(), activity.getName(), categories);
			allActivitiesHeader.add(headerM);
		}

		System.out.println("Todos os servios enviados");

		return allActivitiesHeader;
		
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see wiiser.server.comm.ServerFacade#requestLanguages()
	 */
	public Collection requestLanguages() {
		System.out.println("Todas as linguas...");

		Collection collLanguageModel = new ArrayList();

		Collection collLanguage = facade.searchAllLanguages();
		Iterator iterator = collLanguage.iterator();
		while (iterator.hasNext()) {
			Language language = (Language) iterator.next();
			LanguageModel languageModel = new LanguageModel(language.getId(), language.getName(), 
															getByteArray(language.getFilePath()),
															language.getAcronym());
			collLanguageModel.add(languageModel);
		}
		System.out.println("Todas as linguas enviadas!");

		return collLanguageModel;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see wiiser.server.comm.ServerFacade#requestMap(java.lang.Integer)
	 */
	public MapModel requestMap(Integer languageID) {
		System.out.println("Enviando o mapa...");

		MapModel mapModel = null;
		Map map = facade.searchMap(new Integer(1));

		if (map != null) {
			String mapPath = map.getPhoto();
			String bitmaskPath = map.getBitMask();

			byte[] mapImage = getByteArray(mapPath);
			byte[] bitmaskImage = getByteArray(bitmaskPath);

			/* Add the areas maps with theirs respective photos and coordinates */
			Collection areas = facade.searchAllAreas();
			Collection areaCollection = new ArrayList();
			Iterator iterator = areas.iterator();
			Area area = null;
			AreaModel areaModel = null;
			
			while (iterator.hasNext()) {
				area = (Area) iterator.next();
				areaModel = new AreaModel(area.getId(), area.getName(),
						getByteArray(area.getPhoto()), area.getCoordX(), area
								.getCoordY());
				areaCollection.add(areaModel);
			}
			mapModel = new MapModel(mapImage, bitmaskImage, areaCollection);
		}
		System.out.println("Mapa Enviado!!!");

		return mapModel;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see wiiser.server.comm.ServerFacade#requestAllServices(java.lang.Integer)
	 */
	public Collection requestAllServices(Integer languageID) {
		System.out.println("Procurar todos os servios...");

		Collection allServices = null;
		Service service = null;
		Collection allServicesHeader = new ArrayList();
		HeaderModel headerM = null;
		Collection categories = null;
		Iterator iCategories = null;
		CategoryService categoryService = null;

		allServices = facade.searchServicesByLanguage(languageID);

		Iterator itServices = allServices.iterator();
		while (itServices.hasNext()) {
			service = (Service) itServices.next();
			Collection categoryCollection = facade.searchCategoryServiceByService(service.getId());

			// Categories
			categories = new ArrayList();
			iCategories = categoryCollection.iterator();

			while (iCategories.hasNext()) {
				categoryService = (CategoryService) iCategories.next();
				categories.add(categoryService.getCategory().getName());
			}
			headerM = new HeaderModel(service.getId(), service.getName(), categories);
			allServicesHeader.add(headerM);
		}
		System.out.println("Todos os servios enviados");

		return allServicesHeader;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see wiiser.server.comm.ServerFacade#requestAreaServices(java.lang.Integer,
	 *      java.lang.Integer)
	 */
	public Collection requestAreaServices(Integer rgb, Integer languageID) {
		System.out.println("Servios pela area...");

		Collection collHeadersModel = new ArrayList();
		Collection categories = null;
		Iterator iCategories = null;
		CategoryService categoryService = null;

		Collection areaServices = facade.searchServiceByArea(rgb, languageID);

		Iterator itServices = areaServices.iterator();
		while (itServices.hasNext()) {
			Service service = (Service) itServices.next();
			Collection categoryCollection = facade.searchCategoryServiceByService(service.getId());

			// Categories
			categories = new ArrayList();
			iCategories = categoryCollection.iterator();

			while (iCategories.hasNext()) {
				categoryService = (CategoryService) iCategories.next();
				categories.add(categoryService.getCategory().getName());
			}
			HeaderModel headerM = new HeaderModel(service.getId(), 
												  service.getName(), 
												  categories);

			collHeadersModel.add(headerM);
		}
		System.out.println("Servios pela area enviado!");
		
		return collHeadersModel;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see wiiser.server.comm.ServerFacade#requestServiceInformation(java.lang.Integer,
	 *      java.lang.Integer)
	 */
	public ServiceModel requestServiceInformation(Integer serviceID) {
		System.out.println("Informaes do servio...");

		ServiceModel serviceM = null;
		CategoryService category = null;
		ScheduleService scheduleS = null;
		ScheduleModel scheduleM = null;

		Service service;
		try {
			service = facade.searchService(serviceID);

			if (service != null) {
				service.setPhotoImage(getByteArray(service.getPhoto()));
				
				// Categories
				Collection categoryCollection = facade.searchCategoryServiceByService(service.getId());

				Collection categories = new ArrayList();
				Iterator iterCategory = categoryCollection.iterator();
				while (iterCategory.hasNext()) {
					category = (CategoryService) iterCategory.next();
					categories.add(category.getCategory().getName());
				}
				HeaderModel headerM = new HeaderModel(service.getId(), service.getName(), 
													  categories);

				// Collection scheduleModel
				Collection scheduleCollection = facade.searchScheduleByService(service.getId());
				Collection schedules = new ArrayList();
				Iterator iterator = scheduleCollection.iterator();
				while (iterator.hasNext()) {
					scheduleS = (ScheduleService) iterator.next();
					scheduleM = new ScheduleModel(Constants.dayOfWeekToString(scheduleS.getDayOfWeek()),
												  scheduleS.getHourInitial(), scheduleS.getHourFinal());
					schedules.add(scheduleM);
				}
				serviceM = new ServiceModel(headerM, service.getDescription(),
						service.getPhotoImage(), schedules, service.getValue(),
						null/* getByteArray(s.getXml()) */);
			}
		} catch (NullObjectException e) {
			e.getMessage();
		}
		System.out.println("Informaes dos servios enviadas!!!");

		return serviceM;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see wiiser.server.comm.ServerFacade#requestAllPlaces(java.lang.Integer)
	 */
	public Collection requestAllPlaces(Integer languageID) {
		System.out.println("Todos os lugares...");

		Collection headerCollecion = new ArrayList();
		Collection categories = null;
		Iterator iterCategory = null;
		CategoryPlace category = null;

		try {
		
			Collection collPlaces = this.facade
					.searchPlacesByLanguage(languageID);
			Iterator iPlaces = collPlaces.iterator();

			while (iPlaces.hasNext()) {
				Place place = (Place) iPlaces.next();
				Collection collCategories = this.facade
						.searchCategoryPlaceByPlace(place.getId());

				// Categories
				categories = new ArrayList();
				iterCategory = collCategories.iterator();
				while (iterCategory.hasNext()) {
					category = (CategoryPlace) iterCategory.next();
					categories.add(category.getCategory().getName());
				}

				HeaderModel headerModel = new HeaderModel(place.getId(), place
						.getName(), categories);

				headerCollecion.add(headerModel);
			}

		} catch (NullObjectException e) {
			e.getMessage();
		}
		
		System.out.println("Todos os lugares enviados");
		return headerCollecion;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see wiiser.server.comm.ServerFacade#requestAreaPlaces(java.lang.Integer,
	 *      java.lang.Integer)
	 */
	public Collection requestAreaPlaces(Integer rgb, Integer languageID) {
		System.out.println("Places pela area...");

		Collection collHeaderModel = new ArrayList();
		Collection categories = null;
		Iterator iterCategory = null;
		CategoryPlace category = null;

		try {
			Collection collPlacesAreas = new ArrayList();

			collPlacesAreas = facade.searchPlaceAreasByArea(rgb, languageID);
			Iterator iterator = collPlacesAreas.iterator();
			while (iterator.hasNext()) {
				PlaceArea placeArea = (PlaceArea) iterator.next();
				Place place = placeArea.getPlace();
				Collection collCategoryPlace = facade
						.searchCategoryPlaceByPlace(place.getId());

				// Categories
				categories = new ArrayList();
				iterCategory = collCategoryPlace.iterator();
				while (iterCategory.hasNext()) {
					category = (CategoryPlace) iterCategory.next();
					categories.add(category.getCategory().getName());
				}
				HeaderModel headerModel = new HeaderModel(place.getId(), place
						.getName(), categories);

				collHeaderModel.add(headerModel);
			}

		} catch (NullObjectException e) {
			e.getMessage();
		}
		
		System.out.println("Places pela area enviada!!!");
		return collHeaderModel;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see wiiser.server.comm.ServerFacade#requestPlaceInformation(java.lang.Integer,
	 *      java.lang.Integer, java.lang.Integer)
	 */
	public PlaceModel requestPlaceInformation(Integer placeID, Integer areaID) {
		System.out.println("Informao de um lugar");

		PlaceModel placeModel = null;
		

		try {
			Collection servicesHeader = new ArrayList();
			Collection activitiesHeader = new ArrayList();

			Place place = this.facade.searchPlace(placeID);
			if (place != null) {

				// get the services header
				Collection servicesByPlace = this.facade
						.searchServiceByPlace(placeID);
				Iterator iService = servicesByPlace.iterator();
				while (iService.hasNext()) {
					Service service = (Service) iService.next();
					Collection collCategories = this.facade
							.searchCategoryServiceByService(service.getId());
					// Categories Service
					Collection categories = new ArrayList();
					Iterator iterCategory = collCategories.iterator();
					CategoryService category = null;
					while (iterCategory.hasNext()) {
						category = (CategoryService) iterCategory.next();
						categories.add(category.getCategory().getName());
					}
					HeaderModel header = new HeaderModel(service.getId(),
							service.getName(), categories);
					servicesHeader.add(header);
				}

				// get the activities header
				Collection activitiesByPlace = this.facade
						.searchActivityByPlace(placeID);
				Iterator iActivity = activitiesByPlace.iterator();
				while (iActivity.hasNext()) {
					Activity activity = (Activity) iActivity.next();
					Collection collCategories = this.facade
							.searchCategoryActivityByActivity(activity.getId());
					// Categories Activity
					Collection categories = new ArrayList();
					Iterator iterCategory = collCategories.iterator();
					CategoryActivity category = null;
					while (iterCategory.hasNext()) {
						category = (CategoryActivity) iterCategory.next();
						categories.add(category.getCategory().getName());
					}
					HeaderModel header = new HeaderModel(activity.getId(),
							activity.getName(), categories);
					activitiesHeader.add(header);
				}

				// get placeArea
				Collection coordenadasModel = new ArrayList();
				Collection placeAreas = facade.searchPlaceAreasByPlace(placeID);
				Iterator iPlaceArea = placeAreas.iterator();
				while (iPlaceArea.hasNext()) {
					PlaceArea placeArea = (PlaceArea) iPlaceArea.next();
					coordenadasModel.add(new CoordenadaModel(placeArea
							.getCoordX(), placeArea.getCoordY()));
				}

				// get categoriyPlace
				Collection categories = new ArrayList();
				Collection collCategoryPlace = this.facade
						.searchCategoryPlaceByPlace(placeID);
				Iterator iterCategory = collCategoryPlace.iterator();
				CategoryPlace category = null;
				while (iterCategory.hasNext()) {
					category = (CategoryPlace) iterCategory.next();
					categories.add(category.getCategory().getName());
				}
				HeaderModel headerModel = new HeaderModel(place.getId(), place
						.getName(), categories);
				placeModel = new PlaceModel(headerModel,
						place.getDescription(), getByteArray(place.getPhoto()),
						servicesHeader, activitiesHeader, null/* xml */,
						coordenadasModel);
			}

		} catch (NullObjectException e) {
			e.getMessage();
		}
		
		System.out.println("Informao do lugar enviado");
		return placeModel;
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see wiiser.server.comm.ServerFacade#searchAreaRgb(java.lang.String)
	 */
	public Integer searchAreaRgb(String areaName){
		Integer rgb = null;
		try {
			Area area = facade.searchAreaByName(areaName);
			if(area!=null){
				rgb = area.getId();
			}
		} catch (NullObjectException e) {
			e.printStackTrace();
		}		
		
		return rgb;
	}

	public static void main(String[] args) {
		try {
			ServerCommunication comm = ServerCommunication.getInstance();
			comm.requestActivityInformation(new Integer(1));
			System.out.println("Servidor Pronto!!!");
			while (true)
				;
		} catch (Exception e) {
			e.printStackTrace();
		}

	}

	
}