package com.efetiva.wiiser.server.repository;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.Iterator;

import org.hibernate.Query;
import org.hibernate.Session;

import com.efetiva.wiiser.server.entity.Activity;
import com.efetiva.wiiser.server.entity.Language;
import com.efetiva.wiiser.server.entity.Place;
import com.efetiva.wiiser.server.repository.interfaceRepository.IActivityRepository;
import com.efetiva.wiiser.server.util.hibernate.HibernateUtil;

public class ActivityRepository implements IActivityRepository {

	/***************************************************************************
	 * Constructor of ActivityRepository
	 * 
	 */
	public ActivityRepository() {

	}

	/***************************************************************************
	 * Stores a Activity object in the DB
	 * 
	 * @param activity
	 *            activity to be stored
	 */
	public void register(Activity activity) {

		Session currentSession = HibernateUtil.getCurrentSession();
		currentSession.save(activity);

	}

	/***************************************************************************
	 * Returns a Activity object searched by its identification in the DB.
	 * <p>
	 * If the Activity isn't found it returns null.
	 * 
	 * @param ID
	 *            the identification in the DB
	 * @return a Activity object
	 */
	public Activity search(Integer ID) {
		String query = "select activity.id, activity.name, activity.description, "
				+ "activity.value, activity.photo, activity.place, "
				+ "activity.language, activity.isActive from Activity as "
				+ "activity where activity.id = " + ID;
		Activity activity = null;

		activity = queryActivity(query, activity);
		
		return activity;
	}
		
	/***************************************************************************
	 * Removes a Activity object from the DB
	 * 
	 * @param activity
	 *            activity to be removed
	 */
	public void remove(Activity activity) {

		Session currentSession = (Session) HibernateUtil.getCurrentSession();
		currentSession.delete(activity);

	}

	/***************************************************************************
	 * Updates a Activity object in the DB
	 * 
	 * @param activity
	 *            activity to be updated
	 */
	public void modify(Activity activity) {

		Session currentSession = HibernateUtil.getCurrentSession();
		currentSession.update(activity);

	}

	/**
	 * Searches all existing activities
	 * 
	 * @return a Activity object collection
	 */
	public Collection searchAll() {
		String query = "select activity.id, activity.name, activity.description, "
				+ "activity.value, activity.photo, activity.place, "
				+ "activity.language, activity.isActive from Activity as "
				+ "activity";
		Collection all = new ArrayList();
		
		queryActivitiesCollection(query, all);

		return all;
	}
	
	/**
	 * Searches all existing activities
	 * 
	 * @return a Activity object collection
	 */
	public Collection searchByLanguage(Integer languageID) {
		String query = "select activity.id, activity.name, activity.description, "
				+ "activity.value, activity.photo, activity.place, "
				+ "activity.language, activity.isActive from Activity as "
				+ "activity where activity.language.id = "
				+ languageID
				+ " and activity.isActive = 1";
		Collection all = new ArrayList();
		
		queryActivitiesCollection(query, all);

		return all;
	}

	/***************************************************************************
	 * Returns a Activity object searched by specific date.
	 * <p>
	 * If any Service isn't found it returns an empty collection.
	 * 
	 * @param date
	 *            the specific date
	 * @return a Service object collection
	 */
	public Collection searchByDate(Date date, Integer languageID) {

		String query = "select distinct activity.id, activity.name, activity.description, "
				+ "activity.value, activity.photo, activity.place, "
				+ "activity.language, activity.isActive from Activity as activity, ScheduleActivity as schedule "
				+ "where activity.id = schedule.activity.id and schedule.dayOfWeek = "
				+ date.getDay()
				+ " and activity.isActive = 1"
				+ " and activity.language.id = " + languageID;
		Collection all = new ArrayList();

		queryActivitiesCollection(query, all);

		return all;
	}

	/**
	 * 
	 * @param placeID
	 * @return
	 */
	public Collection searchByArea(Integer areaID, Integer languageID) {
		String query = "select distinct activity.id, activity.name, activity.description, "
				+ "activity.value, activity.photo, activity.place, "
				+ "activity.language, activity.isActive from Activity as activity, PlaceArea as placeArea"
				+ " where activity.place.id = placeArea.place.id"
				+ " and placeArea.area.id = "
				+ areaID
				+ " and activity.isActive = 1"
				+ " and activity.language.id = "
				+ languageID;
		Collection all = new ArrayList();

		queryActivitiesCollection(query, all);

		return all;
	}

	public Collection searchByPlace(Integer placeID) {
		String query = "select distinct activity.id, activity.name, activity.description, "
				+ "activity.value, activity.photo, activity.place, "
				+ "activity.language, activity.isActive from Activity as activity "
				+ "where activity.place.id = "
				+ placeID
				+ " and activity.isActive = 1";
		Collection all = new ArrayList();

		queryActivitiesCollection(query, all);

		return all;
	}

	public Activity searchByName(String name, Integer languageID) {
		String query = "select activity.id, activity.name, activity.description, "
			+ "activity.value, activity.photo, activity.place, "
			+ "activity.language, activity.isActive from Activity as "
			+ "activity where activity.name = " + "'" +name + "' " 
			+ "and activity.language.id = " + languageID;
		Activity activity = null;

		activity = queryActivity(query, activity);
		
		return activity;
	}
	
	/*
	 * metodos privados
	 * metodos extraidos
	 */
	
	private Activity queryActivity(String query, Activity activity) {
		Session currentSession = HibernateUtil.getCurrentSession();
		Query q = currentSession.createQuery(query);
		Collection coll = q.list();
		Iterator iterator = coll.iterator();
		while (iterator.hasNext()) {
			Object obj = (Object) iterator.next();
			Object[] array = (Object[]) obj;
			activity = new Activity((Integer) array[0],
					(String) array[1],(String) array[2], (Double) array[3],
					(String) array[4],(Place) array[5], (Language) array[6],
					(Integer) array[7]);
		}
		return activity;
	}
	
	private void queryActivitiesCollection(String query, Collection all) {
		Session currentSession = HibernateUtil.getCurrentSession();
		Query q = currentSession.createQuery(query);
		Collection coll = q.list();
		Iterator iterator = coll.iterator();
		while (iterator.hasNext()) {
			Object obj = (Object) iterator.next();
			Object[] array = (Object[]) obj;
			Activity activity = new Activity((Integer) array[0],
					(String) array[1], (String) array[2], (Double) array[3],
					(String) array[4], (Place) array[5], (Language) array[6],
					(Integer) array[7]);
			all.add(activity);
		}
	}

	
}
