package com.efetiva.wiiser.server.repository;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import org.hibernate.Query;
import org.hibernate.Session;

import com.efetiva.wiiser.server.entity.Area;
import com.efetiva.wiiser.server.entity.Language;
import com.efetiva.wiiser.server.entity.Map;
import com.efetiva.wiiser.server.repository.interfaceRepository.IAreaRepository;
import com.efetiva.wiiser.server.util.hibernate.HibernateUtil;

public class AreaRepository implements IAreaRepository {

	/***************************************************************************
	 * Constructor of AreaRepository
	 * 
	 */
	public AreaRepository() {

	}

	/***************************************************************************
	 * Stores an Area object in the DB.
	 * 
	 * @param area
	 *            area to be stored
	 */
	public void register(Area area) {

		Session currentSession = HibernateUtil.getCurrentSession();
		currentSession.save(area);

	}

	/***************************************************************************
	 * Returns an Area object searched by its identification in the DB.
	 * <p>
	 * If the Area isn't found it returns null.
	 * 
	 * @param ID
	 *            the identification in the DB
	 * @return an Area object
	 */

	public Area search(Integer ID) {
		String query = "select area.id, area.name, area.description, area.photo, area.map, "
				+ "area.coordX, area.coordY, area.language from Area as area where area.id = "
				+ ID;
		Area area = null;

		area = queryArea(query, area);

		return area;
	}

	/***************************************************************************
	 * Removes an Area object from the DB
	 * 
	 * @param area
	 *            area to be removed
	 */
	public void remove(Area area) {

		Session currentSession = HibernateUtil.getCurrentSession();
		currentSession.delete(area);

	}

	/***************************************************************************
	 * Updates an Area object in the DB
	 * 
	 * @param area
	 *            area to be updated
	 */
	public void modify(Area area) {

		Session currentSession = HibernateUtil.getCurrentSession();
		currentSession.update(area);

	}

	/***************************************************************************
	 * Returns a collection of all Area object stored in the DB.
	 * <p>
	 * If any Area isn't found it returns an empty collection.
	 * 
	 * @return an Area collection
	 */

	public Collection searchAll() {
		String query = "select area.id, area.name, area.description, area.photo, area.map, "
				+ "area.coordX, area.coordY, area.language from Area as area";
		Collection all = new ArrayList();

		queryAreasCollection(query, all);
		return all;
	}
	
	/***************************************************************************
	 * Returns a collection of all Area object with the specific language stored
	 * in the DB.
	 * <p>
	 * If any Area isn't found it returns an empty collection.
	 * 
	 * @return an Area collection
	 */

	public Collection searchByLanguage(Integer languageID) {
		String query = "select area.id, area.name, area.description, area.photo, area.map, "
				+ "area.coordX, area.coordY, area.language from Area as area where area.language.id = "
				+ languageID;
		Collection all = new ArrayList();

		queryAreasCollection(query, all);
		return all;
	}

	public Area searchByName(String name) {
		String query = "select area.id, area.name, area.description, area.photo, area.map, "
				+ "area.coordX, area.coordY, area.language from Area as area where area.name = "
				+ "'" + name + "'";
		Area area = null;

		area = queryArea(query, area);
		return area;
	}
	
	/*
	 * metodos privados 
	 * metodos extraidos
	 */
		
	private Area queryArea(String query, Area area) {
		Session currentSession = HibernateUtil.getCurrentSession();
		Query q = currentSession.createQuery(query);
		Collection coll = q.list();
		Iterator iterator = coll.iterator();
		while (iterator.hasNext()) {
			Object obj = (Object) iterator.next();
			Object[] array = (Object[]) obj;
			area = new Area((Integer) array[0], (String) array[1],
					(String) array[2], (String) array[3], (Map) array[4],
					(Integer) array[5], (Integer) array[6], (Language) array[7]);
		}
		return area;
	}
	
	private void queryAreasCollection(String query, Collection all) {
		Session currentSession = HibernateUtil.getCurrentSession();
		Query q = currentSession.createQuery(query);
		Collection coll = q.list();
		Iterator iterator = coll.iterator();
		while (iterator.hasNext()) {
			Object obj = (Object) iterator.next();
			Object[] array = (Object[]) obj;
			Area area = new Area((Integer) array[0], (String) array[1],
					(String) array[2], (String) array[3], (Map) array[4],
					(Integer) array[5], (Integer) array[6], (Language) array[7]);
			all.add(area);
		}
	}
	
	
}
