package com.efetiva.wiiser.server.repository;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import org.hibernate.Query;
import org.hibernate.Session;

import com.efetiva.wiiser.server.entity.Area;
import com.efetiva.wiiser.server.entity.DynamicInfo;
import com.efetiva.wiiser.server.entity.Language;
import com.efetiva.wiiser.server.repository.interfaceRepository.IDynamicInfoRepository;
import com.efetiva.wiiser.server.util.hibernate.HibernateUtil;

public class DynamicInfoRepository implements IDynamicInfoRepository {

	/***************************************************************************
	 * Constructor of DynamicInfoRepository
	 * 
	 */
	public DynamicInfoRepository() {

	}

	/***************************************************************************
	 * Stores a DynamicInfo object in the DB
	 * 
	 * @param info
	 *            DynamicInfo to be stored
	 */
	public void register(DynamicInfo info) {

		Session currentSession = HibernateUtil.getCurrentSession();
		currentSession.save(info);
	}

	/***************************************************************************
	 * Returns a DynamicInfo object searched by its identification in the DB.
	 * <p>
	 * If the DynamicInfo isn't found it returns null.
	 * 
	 * @param ID
	 *            the identification in the DB
	 * @return a DynamicInfo object
	 */
	public DynamicInfo search(Integer ID) {
		String query = "select dynamicInfo.id,  dynamicInfo.description, dynamicInfo.language from DynamicInfo"
				+ " as dynamicInfo where dynamicInfo.id = " + ID;
		DynamicInfo info = null;

		Session currentSession = HibernateUtil.getCurrentSession();
		Query q = currentSession.createQuery(query);
		Collection coll = q.list();
		Iterator iterator = coll.iterator();
		while (iterator.hasNext()) {
			Object obj = (Object) iterator.next();
			Object[] array = (Object[]) obj;
			info = new DynamicInfo((Integer) array[0], (String) array[1],
					(Language) array[2]);
		}
		return info;
	}

	/***************************************************************************
	 * Removes a DynamicInfo object from the DB
	 * 
	 * @param dynamicInfo
	 *            DynamicInfo to be removed
	 */
	public void remove(DynamicInfo dynamicInfo) {

		Session currentSession = HibernateUtil.getCurrentSession();
		currentSession.delete(dynamicInfo);
	}

	/***************************************************************************
	 * Updates a DynamicInfo object in the DB
	 * 
	 * @param info
	 *            DynamicInfo to be updated
	 */
	public void modify(DynamicInfo info) {

		Session currentSession = HibernateUtil.getCurrentSession();
		currentSession.update(info);
	}

	/***************************************************************************
	 * Returns a collection of all DynamicInfo object stored in the DB.
	 * <p>
	 * If any DynamicInfo isn't found it returns an empty collection.
	 * 
	 * @return a DynamicInfo collection
	 */
	public Collection searchAll() {
		String query = "select distinct dynamicInfo.id, dynamicInfo.description, "
				+ "dynamicInfo.language, schedule.area from DynamicInfo as dynamicInfo, ScheduleInfo as schedule where schedule.dynamicInfo.id = dynamicInfo.id";
		Collection all = new ArrayList();
		Session currentSession = HibernateUtil.getCurrentSession();
		Query q = currentSession.createQuery(query);
		Collection coll = q.list();
		Iterator iterator = coll.iterator();
		while (iterator.hasNext()) {
			Object obj = (Object) iterator.next();
			Object[] array = (Object[]) obj;
			DynamicInfo info = new DynamicInfo((Integer) array[0],
					(String) array[1], (Language) array[2], (Area) array[3]);
			all.add(info);
		}
		return all;
	}

	/***************************************************************************
	 * Returns a collection of all DynamicInfo object stored in the DB with the
	 * specific languageID.
	 * <p>
	 * If any DynamicInfo isn't found it returns an empty collection.
	 * 
	 * @return a DynamicInfo collection
	 */
	public Collection searchByLanguage(Integer languageID) {
		String query = "select dynamicInfo.id, dynamicInfo.description, "
				+ "dynamicInfo.language from DynamicInfo as dynamicInfo"
				+ " where dynamicInfo.language.id = " + languageID;
		Collection all = new ArrayList();
		Session currentSession = HibernateUtil.getCurrentSession();
		Query q = currentSession.createQuery(query);
		Collection coll = q.list();
		Iterator iterator = coll.iterator();
		while (iterator.hasNext()) {
			Object obj = (Object) iterator.next();
			Object[] array = (Object[]) obj;
			DynamicInfo info = new DynamicInfo((Integer) array[0],
					(String) array[1], (Language) array[2]);
			all.add(info);
		}
		return all;
	}

	public DynamicInfo selectMaxId() {
		DynamicInfo info = null;
		String query = "select dynamicInfo.id, dynamicInfo.description, "
				+ "dynamicInfo.language from DynamicInfo as dynamicInfo"
				+ " where dynamicInfo.id = (select MAX(dynamicInfo2.id) from DynamicInfo as dynamicInfo2)";

		Session currentSession = HibernateUtil.getCurrentSession();
		Query q = currentSession.createQuery(query);
		Collection coll = q.list();
		Iterator iterator = coll.iterator();
		if (iterator.hasNext()) {
			Object obj = (Object) iterator.next();
			Object[] array = (Object[]) obj;
			info = new DynamicInfo((Integer) array[0], (String) array[1],
					(Language) array[2]);
		}

		return info;
	}
	
}
