package com.efetiva.wiiser.server.repository;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import org.hibernate.Query;
import org.hibernate.Session;

import com.efetiva.wiiser.server.entity.Language;
import com.efetiva.wiiser.server.entity.Place;
import com.efetiva.wiiser.server.repository.interfaceRepository.IPlaceRepository;
import com.efetiva.wiiser.server.util.hibernate.HibernateUtil;

public class PlaceRepository implements IPlaceRepository {

	/***************************************************************************
	 * Constructor of PlaceRepository
	 * 
	 */
	public PlaceRepository() {

	}

	/***************************************************************************
	 * Stores a Place object in the DB
	 * 
	 * @param place
	 *            place to be stored
	 */
	public void register(Place place) {

		Session currentSession = HibernateUtil.getCurrentSession();
		currentSession.save(place);
	}

	/***************************************************************************
	 * Returns a Place object searched by its identification in the DB.
	 * <p>
	 * If the Place isn't found it returns null.
	 * 
	 * @param ID
	 *            the identification in the DB
	 * @return a Place object
	 */
	public Place search(Integer ID) {
		String query = "select place.id, place.name, place.description, "
				+ "place.photo, place.language from Place as "
				+ "place where place.id = " + ID;
		Place place = null;
		
		place = queryPlace(query, place);
		
		return place;
	}
	
	/***************************************************************************
	 * Removes a Place object from the DB
	 * 
	 * @param place
	 *            place to be removed
	 */
	public void remove(Place place) {

		Session currentSession = HibernateUtil.getCurrentSession();
		currentSession.delete(place);

	}

	/***************************************************************************
	 * Updates a Place object in the DB
	 * 
	 * @param place
	 *            place to be updated
	 */
	public void modify(Place place) {

		Session currentSession = HibernateUtil.getCurrentSession();
		currentSession.update(place);

	}

	/***************************************************************************
	 * Returns a collection of all Place object stored in the DB.
	 * <p>
	 * If any Place isn't found it returns an empty collection.
	 * 
	 * @return a Place collection
	 */
	public Collection searchAll() {
		String query = "select place.id, place.name, place.description, "
				+ "place.photo, place.language from Place as place";
		Collection all = new ArrayList();

		queryPlacesCollection(query, all);
		
		return all;
	}
	
	/***************************************************************************
	 * Returns a collection of all Place object stored in the DB.
	 * <p>
	 * If any Place isn't found it returns an empty collection.
	 * 
	 * @return a Place collection
	 */
	public Collection searchByLanguage(Integer languageID) {
		String query = "select place.id, place.name, place.description, "
				+ "place.photo, place.language from Place as place "
				+ "where place.language = " + languageID;
		Collection all = new ArrayList();

		queryPlacesCollection(query, all);
		
		return all;
	}

	/***************************************************************************
	 * Returns a Place object searched by its name in the DB.
	 * <p>
	 * If the Place isn't found it returns null.
	 * 
	 * @param placeName
	 *            the iname in the DB
	 * @return a Place object
	 */
	public Place searchByName(String placeName, Integer languageID) {
		String query = "select place.id, place.name, place.description, "
				+ "place.photo, place.language from Place as "
				+ "place where place.name = " + "'" + placeName + "' " 
				+ "and place.language.id = " + languageID ;
		Place place = null;
		
		place = queryPlace(query, place);
		
		return place;
	}
	
	/*
	 * metodos privados
	 * metodos extraidos
	 */
	
	private Place queryPlace(String query, Place place) {
		Session currentSession = HibernateUtil.getCurrentSession();
		Query q = currentSession.createQuery(query);
		Collection coll = q.list();
		Iterator iterator = coll.iterator();
		while (iterator.hasNext()) {
			Object obj = (Object) iterator.next();
			Object[] array = (Object[]) obj;
			place = new Place((Integer) array[0], (String) array[1],
					(String) array[2], (String) array[3], (Language) array[4]);
		}
		return place;
	}
	
	private void queryPlacesCollection(String query, Collection all) {
		Session currentSession = HibernateUtil.getCurrentSession();
		Query q = currentSession.createQuery(query);
		Collection coll = q.list();
		Iterator iterator = coll.iterator();
		while (iterator.hasNext()) {
			Object obj = (Object) iterator.next();
			Object[] array = (Object[]) obj;
			Place place = new Place((Integer) array[0], (String) array[1],
					(String) array[2], (String) array[3], (Language) array[4]);
			all.add(place);
		}
	}
	
}
