package com.efetiva.wiiser.server.repository;

import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.Iterator;

import org.hibernate.Query;
import org.hibernate.Session;

import com.efetiva.wiiser.server.entity.Activity;
import com.efetiva.wiiser.server.entity.ScheduleActivity;
import com.efetiva.wiiser.server.repository.interfaceRepository.IScheduleActivityRepository;
import com.efetiva.wiiser.server.util.hibernate.HibernateUtil;

public class ScheduleActivityRepository implements IScheduleActivityRepository {

	/***************************************************************************
	 * Constructor of ScheduleActivityRepository
	 * 
	 */
	public ScheduleActivityRepository() {

	}

	/***************************************************************************
	 * Updates a ScheduleActivity object in the DB
	 * 
	 * @param schedule
	 *            ScheduleActivity to be updated
	 */
	public void modify(ScheduleActivity schedule) {

		Session currentSession = HibernateUtil.getCurrentSession();
		currentSession.update(schedule);

	}

	/***************************************************************************
	 * Stores a ScheduleActivity object in the DB
	 * 
	 * @param schedule
	 *            ScheduleActivity to be stored
	 */
	public void register(ScheduleActivity schedule) {

		Session currentSession = HibernateUtil.getCurrentSession();
		currentSession.save(schedule);

	}

	/***************************************************************************
	 * Removes a ScheduleActivity object from the DB
	 * 
	 * @param schedule
	 *            ScheduleActivity to be removed
	 */
	public void remove(ScheduleActivity schedule) {

		Session currentSession = HibernateUtil.getCurrentSession();
		currentSession.delete(schedule);
	}

	/**
	 * Searches all existing ScheduleActivity
	 * 
	 * @return a ScheduleActivity object collection
	 */
	public Collection searchAll() {
		String query = "select schedule.id, schedule.activity, schedule.dayOfWeek, schedule.hour "
				+ "from ScheduleActivity as schedule";
		Collection all = new ArrayList();
		
		queryScheduleActivitiesCollection(query, all);

		return all;
	}
	
	/***************************************************************************
	 * Returns a collection of ScheduleActivity object searched by
	 * identification in the DB.
	 * <p>
	 * If any ScheduleActivity isn't found it returns an empty collection.
	 * 
	 * @param ID
	 *            activity identification in the DB
	 * @return a ScheduleActivity object
	 */
	public Collection searchByActivity(Integer ID) {
		String query = "select schedule.id, schedule.activity, schedule.dayOfWeek, schedule.hour "
				+ "from ScheduleActivity as schedule where schedule.activity.id = "
				+ ID;
		Collection all = new ArrayList();
		
		queryScheduleActivitiesByActivitiesCollection(query, all);

		return all;
	}
	
	/***************************************************************************
	 * Returns a collection of ScheduleActivity object searched by name in the
	 * DB.
	 * <p>
	 * If any ScheduleActivity isn't found it returns an empty collection.
	 * 
	 * @param placeName
	 *            activity name in the DB
	 * @return a ScheduleActivity object
	 */
	public Collection searchByActivityName(String namePlace, Integer language) {
		String query = "select schedule.id, schedule.activity, schedule.dayOfWeek, schedule.hour "
				+ "from ScheduleActivity as schedule where schedule.activity.name = '"
				+ namePlace
				+ "' and schedule.activity.language.id = "
				+ language;
		Collection all = new ArrayList();
		
		queryScheduleActivitiesCollection(query, all);

		return all;
	}

	public void removeByActivity(Integer activityID) {
		String query = "delete from ScheduleActivity as scheduleActivity where scheduleActivity.activity.id = "
				+ activityID;
		Session currentSession = HibernateUtil.getCurrentSession();
		currentSession.createQuery(query);
	}

	public ScheduleActivity search(Integer scheduleID) {
		String query = "select schedule.id, schedule.activity, schedule.dayOfWeek, schedule.hour "
				+ "from ScheduleActivity as schedule where schedule.id = "
				+ scheduleID;
		ScheduleActivity schedule = null;
		
		schedule = queryScheduleActivity(query, schedule);

		return schedule;
	}
		
	/*
	 * metodos privados
	 * metodos extraidos
	 */
	
	private void queryScheduleActivitiesCollection(String query, Collection all) {
		Session currentSession = HibernateUtil.getCurrentSession();
		Query q = currentSession.createQuery(query);
		Collection coll = q.list();
		Iterator iterator = coll.iterator();
		while (iterator.hasNext()) {
			Object obj = (Object) iterator.next();
			Object[] array = (Object[]) obj;
			ScheduleActivity scheduleActivity = new ScheduleActivity(
					(Integer) array[0], (Activity) array[1],
					(Integer) array[2], (Date) array[3]);
			all.add(scheduleActivity);
		}
	}
	
	private void queryScheduleActivitiesByActivitiesCollection(String query,
			Collection all) {
		Session currentSession = HibernateUtil.getCurrentSession();
		Query q = currentSession.createQuery(query);
		Collection coll = q.list();
		Iterator iterator = coll.iterator();
		while (iterator.hasNext()) {
			Object obj = (Object) iterator.next();
			Object[] array = (Object[]) obj;
			Date d = new Date();
			Timestamp t = (Timestamp) array[3];
			d.setHours(t.getHours());
			d.setMinutes(t.getMinutes());
			ScheduleActivity scheduleActivity = new ScheduleActivity(
					(Integer) array[0], (Activity) array[1],
					(Integer) array[2], d);
			all.add(scheduleActivity);
		}
	}
	
	private ScheduleActivity queryScheduleActivity(String query,
			ScheduleActivity schedule) {
		Session currentSession = HibernateUtil.getCurrentSession();
		Query q = currentSession.createQuery(query);
		Collection coll = q.list();
		Iterator iterator = coll.iterator();
		while (iterator.hasNext()) {
			Object obj = (Object) iterator.next();
			Object[] array = (Object[]) obj;
			schedule = new ScheduleActivity((Integer) array[0],
					(Activity) array[1], (Integer) array[2], (Date) array[3]);
		}
		return schedule;
	}
	
}
