package com.efetiva.wiiser.server.repository;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.Iterator;

import org.hibernate.Query;
import org.hibernate.Session;

import com.efetiva.wiiser.server.entity.Area;
import com.efetiva.wiiser.server.entity.DynamicInfo;
import com.efetiva.wiiser.server.entity.ScheduleInfo;
import com.efetiva.wiiser.server.repository.interfaceRepository.IScheduleInfoRepository;
import com.efetiva.wiiser.server.util.hibernate.HibernateUtil;

public class ScheduleInfoRepository implements IScheduleInfoRepository {

	/***************************************************************************
	 * Constructor of ScheduleInfoRepository
	 * 
	 */
	public ScheduleInfoRepository() {

	}

	/***************************************************************************
	 * Updates a ScheduleInfo object in the DB
	 * 
	 * @param schedule
	 *            ScheduleInfo to be updated
	 */
	public void modify(ScheduleInfo schedule) {
		Session currentSession = HibernateUtil.getCurrentSession();
		currentSession.update(schedule);

	}

	/***************************************************************************
	 * Stores a ScheduleInfo object in the DB
	 * 
	 * @param schedule
	 *            ScheduleInfo to be stored
	 */
	public void register(ScheduleInfo schedule) {
		Session currentSession = HibernateUtil.getCurrentSession();
		currentSession.save(schedule);
	}

	/***************************************************************************
	 * Removes a ScheduleInfo object from the DB
	 * 
	 * @param schedule
	 *            ScheduleInfo to be removed
	 */
	public void remove(ScheduleInfo schedule) {

		Session currentSession = HibernateUtil.getCurrentSession();
		currentSession.delete(schedule);
	}

	/**
	 * Searches all existing ScheduleInfo
	 * 
	 * @return a ScheduleInfo object collection
	 */
	public Collection searchAll() {
		String query = "select scheduleInfo.id, scheduleInfo.dynamicInfo, scheduleInfo.area, "
				+ "scheduleInfo.dayOfWeek, scheduleInfo.hour "
				+ "from ScheduleInfo as scheduleInfo";

		Collection all = new ArrayList();

		queryScheduleInfoCollection(query, all);
		
		return all;
	}
	
	/**
	 * Searches all existing ScheduleInfo with Specific DynamicInfo id
	 * 
	 * @return a ScheduleInfo object collection
	 */
	public Collection searchByDynamicInfo(Integer id) {
		String query = "select schedule.id, schedule.dynamicInfo, schedule.area, schedule.dayOfWeek, schedule.hour "
				+ "from ScheduleInfo as schedule where schedule.dynamicInfo.id = "
				+ id;
		Collection all = new ArrayList();
		
		queryScheduleInfoCollection(query, all);

		return all;
	}

	public void removeByDynamicInfo(Integer infoID) {
		String query = "delete from ScheduleInfo as scheduleInfo "
				+ "where scheduleInfo.dynamicInfo.id = " + infoID;
		Session currentSession = HibernateUtil.getCurrentSession();
		currentSession.createQuery(query);

	}

	public Collection searchByHour(Date date) {
		String dateString = hourToString(date);
		String query = "select schedule.id, schedule.dynamicInfo, schedule.area, schedule.dayOfWeek, schedule.hour "
				+ "from ScheduleInfo as schedule where schedule.dayOfWeek = "
				+ date.getDay() + " and schedule.hour = " + "'" +dateString+ "'";
		Collection all = new ArrayList();
		
		queryScheduleInfoCollection(query, all);

		return all;
	}

	public static String hourToString(Date date) {
		String dateString = new String();
		String minutes = "" + date.getMinutes();
		String hours = "" + date.getHours();
		if (date.getHours() < 10) {
			hours = "0" + date.getHours();
		}
		if (date.getMinutes() < 10) {
			minutes = "0" + date.getMinutes();
		}
		dateString = hours + minutes;
		System.out.println(dateString);
		return dateString;
	}
	
	/*
	 * metodos privados
	 * metodos extraidos
	 */
	
	private void queryScheduleInfoCollection(String query, Collection all) {
		Session currentSession = HibernateUtil.getCurrentSession();
		Query q = currentSession.createQuery(query);
		Collection coll = q.list();
		Iterator iterator = coll.iterator();
		while (iterator.hasNext()) {
			Object obj = (Object) iterator.next();
			Object[] array = (Object[]) obj;
			ScheduleInfo scheduleInfo = new ScheduleInfo((Integer) array[0],
					(DynamicInfo) array[1], (Area) array[2],
					(Integer) array[3], (String) array[4]);
			all.add(scheduleInfo);
		}
	}
	
}
