package com.efetiva.wiiser.server.repository;

import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.Iterator;

import org.hibernate.Query;
import org.hibernate.Session;

import com.efetiva.wiiser.server.entity.ScheduleService;
import com.efetiva.wiiser.server.entity.Service;
import com.efetiva.wiiser.server.repository.interfaceRepository.IScheduleServiceRepository;
import com.efetiva.wiiser.server.util.hibernate.HibernateUtil;

public class ScheduleServiceRepository implements IScheduleServiceRepository {

	/***************************************************************************
	 * Constructor of ScheduleServiceRepository
	 * 
	 */
	public ScheduleServiceRepository() {

	}

	/***************************************************************************
	 * Updates a ScheduleService object in the DB
	 * 
	 * @param schedule
	 *            ScheduleService to be updated
	 */
	public void modify(ScheduleService schedule) {

		Session currentSession = HibernateUtil.getCurrentSession();
		currentSession.update(schedule);

	}

	/***************************************************************************
	 * Stores a ScheduleService object in the DB
	 * 
	 * @param schedule
	 *            ScheduleService to be stored
	 */
	public void register(ScheduleService schedule) {

		Session currentSession = HibernateUtil.getCurrentSession();
		currentSession.save(schedule);

	}

	/***************************************************************************
	 * Removes a ScheduleService object from the DB
	 * 
	 * @param schedule
	 *            ScheduleService to be removed
	 */
	public void remove(ScheduleService schedule) {

		Session currentSession = HibernateUtil.getCurrentSession();
		currentSession.delete(schedule);
	}

	/**
	 * Searches all existing ScheduleService
	 * 
	 * @return a ScheduleService object collection
	 */
	public Collection searchAll() {
		String query = "select schedule.id, schedule.service, schedule.dayOfWeek, schedule.hourInitial, schedule.hourFinal "
				+ "from ScheduleService as schedule";
		Collection all = new ArrayList();

		queryScheduleServicesCollection(query, all);

		return all;
	}
	
	/***************************************************************************
	 * Returns a collection of ScheduleService object searched by service
	 * identification in the DB.
	 * <p>
	 * If any ScheduleService isn't found it returns an empty collection.
	 * 
	 * @param ID
	 *            service identification in the DB
	 * @return a ScheduleService object
	 */
	public Collection searchByService(Integer ID) {
		String query = "select schedule.id, schedule.service, schedule.dayOfWeek, schedule.hourInitial, schedule.hourFinal "
				+ "from ScheduleService as schedule where schedule.service.id = "
				+ ID;
		Collection all = new ArrayList();
		
		queryScheduleServicesByServiceCollection(query, all);

		return all;
	}
	
	public void removeByService(Integer serviceID) {
		String query = "delete from ScheduleService as scheduleService where scheduleService.service.id = "
				+ serviceID;
		Session currentSession = HibernateUtil.getCurrentSession();
		currentSession.createQuery(query);

	}

	public ScheduleService search(Integer id){
		ScheduleService schedule = null;
		if(id!=null){
			String query = "select schedule.id, schedule.service, schedule.dayOfWeek, schedule.hourInitial," 
				+ " schedule.hourFinal from ScheduleService as schedule where schedule.id = "
				+ id;
			
			schedule = queryScheduleService(schedule, query);
		}
		return schedule;
	}
		
	/*
	 * metodos privados
	 * metodos extraidos
	 */
	
	private void queryScheduleServicesCollection(String query, Collection all) {
		Session currentSession = HibernateUtil.getCurrentSession();
		Query q = currentSession.createQuery(query);
		Collection coll = q.list();
		Iterator iterator = coll.iterator();
		while (iterator.hasNext()) {
			Object obj = (Object) iterator.next();
			Object[] array = (Object[]) obj;
			ScheduleService scheduleService = new ScheduleService(
					(Integer) array[0], (Service) array[1], (Integer) array[2],
					(Date) array[3], (Date) array[4]);
			all.add(scheduleService);
		}
	}
	
	private void queryScheduleServicesByServiceCollection(String query,
			Collection all) {
		Session currentSession = HibernateUtil.getCurrentSession();
		Query q = currentSession.createQuery(query);
		Collection coll = q.list();
		Iterator iterator = coll.iterator();
		while (iterator.hasNext()) {
			Object obj = (Object) iterator.next();
			Object[] array = (Object[]) obj;
			Date d1 = new Date();
			Timestamp t = (Timestamp) array[3];
			d1.setHours(t.getHours());
			d1.setMinutes(t.getMinutes());
			Date d2 = new Date();
			t = (Timestamp) array[4];
			d2.setHours(t.getHours());
			d2.setMinutes(t.getMinutes());
			ScheduleService scheduleService = new ScheduleService(
					(Integer) array[0], (Service) array[1], (Integer) array[2],
					d1, d2);
			all.add(scheduleService);
		}
	}
	
	private ScheduleService queryScheduleService(ScheduleService schedule,
			String query) {
		Session currentSession = HibernateUtil.getCurrentSession();
		Query q = currentSession.createQuery(query);
		Collection coll = q.list();
		Iterator iterator = coll.iterator();
		while (iterator.hasNext()) {
			Object obj = (Object) iterator.next();
			Object[] array = (Object[]) obj;
			schedule = new ScheduleService((Integer) array[0],
					(Service) array[1], (Integer) array[2], (Date) array[3], (Date) array[4]) ;
		}
		return schedule;
	}
	
}
