package com.efetiva.wiiser.server.repository;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.Iterator;

import org.hibernate.Query;
import org.hibernate.Session;

import com.efetiva.wiiser.server.entity.Language;
import com.efetiva.wiiser.server.entity.Place;
import com.efetiva.wiiser.server.entity.Service;
import com.efetiva.wiiser.server.repository.interfaceRepository.IServiceRepository;
import com.efetiva.wiiser.server.util.hibernate.HibernateUtil;

public class ServiceRepository implements IServiceRepository {

	/***************************************************************************
	 * Constructor of ServiceRepository
	 * 
	 */
	public ServiceRepository() {

	}

	/***************************************************************************
	 * Stores a Service object in the DB
	 * 
	 * @param service
	 *            service to be stored
	 */
	public void register(Service service) {

		Session currentSession = HibernateUtil.getCurrentSession();
		currentSession.save(service);

	}

	/***************************************************************************
	 * Returns a Service object searched by its identification in the DB.
	 * <p>
	 * If the Service isn't found it returns null.
	 * 
	 * @param ID
	 *            the identification in the DB
	 * @return a Service object
	 */
	public Service search(Integer ID) {
		String query = "select service.id, service.name, service.description, "
				+ "service.value, service.photo, service.place, service.isActive, "
				+ "service.language from Service as service where service.id = "
				+ ID;
		Service service = null;

		service = queryService(query, service);

		return service;
	}
	
	/***************************************************************************
	 * Removes a Service object from the DB
	 * 
	 * @param service
	 *            service to be removed
	 */
	public void remove(Service service) {

		Session currentSession = (Session) HibernateUtil.getCurrentSession();
		currentSession.delete(service);

	}

	/***************************************************************************
	 * Updates a Service object in the DB
	 * 
	 * @param service
	 *            service to be updated
	 */
	public void modify(Service service) {

		Session currentSession = HibernateUtil.getCurrentSession();
		currentSession.update(service);

	}

	/**
	 * Searches all existing services
	 * 
	 * @return a Service object collection
	 */
	public Collection searchAllServices() {
		String query = "select service.id, service.name, service.description, "
				+ "service.value, service.photo, service.place, service.isActive, "
				+ "service.language from Service as service";
		Collection all = new ArrayList();
		
		queryServicesCollection(query, all);

		return all;
	}
	
	/**
	 * Searches all existing services with the specific language
	 * 
	 * @return a Service object collection
	 */
	public Collection searchServicesByLanguage(Integer languageID) {
		String query = "select service.id, service.name, service.description, "
				+ "service.value, service.photo, service.place, service.isActive, "
				+ "service.language from Service as service where service.language.id "
				+ "= " + languageID;
		Collection all = new ArrayList();
		
		queryServicesCollection(query, all);

		return all;
	}

	/***************************************************************************
	 * Returns a Service object searched by specific date.
	 * <p>
	 * If any Service isn't found it returns an empty collection.
	 * 
	 * @param date
	 *            the specific date
	 * @return a Service object collection
	 */
	public Collection searchServiceByDate(Date date) {
		String query = "select distinct service.id, service.name, service.description, "
				+ "service.value, service.photo, service.place, service.isActive, "
				+ "service.language from Service as service, ScheduleService as schedule "
				+ "where service.id = schedule.service.id and schedule.dayOfWeek = "
				+ date.getDay();
		Collection all = new ArrayList();
		
		queryServicesCollection(query, all);

		return all;
	}

	public Collection searchServiceByPlace(Integer placeID) {
		String query = "select distinct service.id, service.name, service.description, "
				+ "service.value, service.photo, service.place, service.isActive, "
				+ "service.language from Service as service where service.place.id = "
				+ placeID + " and service.isActive = 1";
		Collection all = new ArrayList();
		
		queryServicesCollection(query, all);

		return all;
	}

	public Collection searchServiceByArea(Integer rgb, Integer languageID) {
		String query = "select distinct service.id, service.name, service.description, "
				+ "service.value, service.photo, service.place, service.isActive, "
				+ "service.language from Service as service, PlaceArea as placeArea"
				+ " where service.place.id = placeArea.place.id "
				+ " and placeArea.area.id = "
				+ rgb
				+ " and service.isActive = 1 and "
				+ "service.language.id = "
				+ languageID;
		Collection all = new ArrayList();
		
		queryServicesCollection(query, all);
		
		return all;
	}

	public Service searchByName(String name, Integer languageID) {
		String query = "select service.id, service.name, service.description, "
				+ "service.value, service.photo, service.place, service.isActive, "
				+ "service.language from Service as service where service.name = '"
				+ name + "'" + " and service.language.id = " + languageID ;
		Service service = null;

		service = queryService(query, service);

		return service;
	}
	
	/*
	 * metodos privados
	 * metodos extraidos
	 */
	
	private Service queryService(String query, Service service) {
		Session currentSession = HibernateUtil.getCurrentSession();
		Query q = currentSession.createQuery(query);
		Collection coll = q.list();
		Iterator iterator = coll.iterator();
		while (iterator.hasNext()) {
			Object obj = (Object) iterator.next();
			Object[] array = (Object[]) obj;
			service = new Service((Integer) array[0], (String) array[1],
					(String) array[2], (Double) array[3], (String) array[4],
					(Place) array[5], (Integer) array[6], (Language) array[7]);
		}
		return service;
	}
	
	private void queryServicesCollection(String query, Collection all) {
		Session currentSession = HibernateUtil.getCurrentSession();
		Query q = currentSession.createQuery(query);
		Collection coll = q.list();
		Iterator iterator = coll.iterator();
		while (iterator.hasNext()) {
			Object obj = (Object) iterator.next();
			Object[] array = (Object[]) obj;
			Service service = new Service((Integer) array[0],
					(String) array[1], (String) array[2], (Double) array[3],
					(String) array[4], (Place) array[5], (Integer) array[6],
					(Language) array[7]);
			all.add(service);
		}
	}

}
