package com.efetiva.wiiser.server.repository.interfaceBusiness;

import java.util.Collection;


import com.efetiva.wiiser.server.entity.Area;
import com.efetiva.wiser.exception.ExistingObjectException;
import com.efetiva.wiser.exception.NonExistingObjectException;
import com.efetiva.wiser.exception.NullObjectException;

public interface IAreaControl {

	/***************************************************************************
	 * Stores an Area object in the DB.
	 * 
	 * @param area
	 *            area to be stored
	 * @throws ExistingObjectException
	 * @throws NullObjectException
	 */
	public void register(Area area) throws ExistingObjectException,
			NullObjectException;

	/***************************************************************************
	 * Returns an Area object searched by its identification in the DB.
	 * 
	 * @param ID
	 *            the identification in the DB
	 * @return an Area object
	 * @throws NullObjectException 
	 */
	public Area search(Integer ID) throws NullObjectException;

	/***************************************************************************
	 * Removes an Area object from the DB
	 * 
	 * @param area
	 *            area to be removed
	 * @throws NonExistingObjectException
	 * @throws NullObjectException
	 */
	public void remove(Area area) throws NonExistingObjectException,
			NullObjectException;

	/***************************************************************************
	 * Updates an Area object in the DB
	 * 
	 * @param area
	 *            area to be updated
	 * @throws NonExistingObjectException
	 * @throws NullObjectException
	 */
	public void modify(Area area) throws NonExistingObjectException,
			NullObjectException;

	/***************************************************************************
	 * Returns a collection of all Area object stored in the DB.
	 * 
	 * @return an Area collection
	 */
	public Collection searchAll();

	/***************************************************************************
	 * Returns a collection of all Area object stored in the DB with the
	 * specific language.
	 * 
	 * @return an Area collection
	 * @throws NullObjectException 
	 */
	public Collection searchByLanguage(Integer languageID) throws NullObjectException;

	/**
	 * Search area with specific name
	 * 
	 * @param name
	 * @return
	 * @throws NullObjectException 
	 */
	public Area searchByName(String name) throws NullObjectException;

}