package com.efetiva.wiiser.server.repository.interfaceBusiness;

import java.util.Collection;


import com.efetiva.wiiser.server.entity.CategoryActivity;
import com.efetiva.wiser.exception.ExistingObjectException;
import com.efetiva.wiser.exception.NonExistingObjectException;
import com.efetiva.wiser.exception.NullObjectException;

public interface ICategoryActivityControl {

	/***************************************************************************
	 * Stores a categoryActivity object in the DB
	 * 
	 * @param categoryActivity
	 *            categoryActivity to be stored
	 * @throws ExistingObjectException 
	 * @throws NullObjectException 
	 */
	public void register(CategoryActivity categoryActivity) throws ExistingObjectException, NullObjectException;

	/***************************************************************************
	 * Returns a categoryActivity object searched by its identification in the
	 * DB.
	 * <p>
	 * If the categoryActivity isn't found it returns null.
	 * 
	 * @param categoryID, activityID
	 *            the identification in the DB
	 * @return a categoryActivity object
	 * @throws NullObjectException 
	 */
	public CategoryActivity search(Integer categoryID, Integer activityID) throws NullObjectException;

	/***************************************************************************
	 * Removes a categoryActivity object from the DB
	 * 
	 * @param categoryActivity
	 *            categoryActivity to be removed
	 * @throws NonExistingObjectException 
	 * @throws NullObjectException 
	 */
	public void remove(CategoryActivity categoryActivity) throws NonExistingObjectException, NullObjectException;

	/***************************************************************************
	 * Returns a collection of all categoryActivity object stored in the DB.
	 * <p>
	 * If any categoryActivity isn't found it returns an empty collection.
	 * 
	 * @return a categoryActivity collection
	 */
	public Collection searchAll();
	
	/***************************************************************************
	 * Returns a collection of all categoryActivity object stored in the DB
	 * with the specific language.	  
	 * <p>
	 * If any categoryActivity isn't found it returns an empty collection.
	 * 
	 * @param languageID
	 * @return a categoryActivity collection
	 * @throws NullObjectException 
	 */
	public Collection searchByLanguage(Integer languageID) throws NullObjectException;
	
	/***************************************************************************
	 * Returns a collection of all categoryActivity object stored in the DB with the
	 * 			specific Activity ID.
	 * <p>
	 * If any categoryActivity isn't found it returns an empty collection.
	 * 
	 * @return a categoryActivity collection
	 * @throws NullObjectException 
	 */
	public Collection searchByActivity(Integer activityID) throws NullObjectException;
	
	/***************************************************************************
	 * Removes a categoryActivity object from the DB 
	 * with the specific activityID
	 * 
	 * @param activityID
	 * @throws NullObjectException 
	 *            
	 * @throws NonExistingObjectException 
	 * @throws NullObjectException 
	 */
	public void removeByActivity(Integer activityID) throws NullObjectException;
}
