package com.efetiva.wiiser.server.repository.interfaceBusiness;

import java.util.Collection;



import com.efetiva.wiiser.server.entity.Category;
import com.efetiva.wiser.exception.ExistingObjectException;
import com.efetiva.wiser.exception.NonExistingObjectException;
import com.efetiva.wiser.exception.NullObjectException;

public interface ICategoryControl {

	/***************************************************************************
	 * Stores a category object in the DB
	 * 
	 * @param category
	 *            category to be stored
	 * @throws ExistingObjectException 
	 * @throws NullObjectException 
	 */
	public void register(Category category) throws ExistingObjectException, NullObjectException;

	/***************************************************************************
	 * Returns a category object searched by its identification in the DB.
	 * <p>
	 * If the category isn't found it returns null.
	 * 
	 * @param ID
	 *            the identification in the DB
	 * @return a category object
	 * @throws NullObjectException 
	 */
	public Category search(Integer ID) throws NullObjectException;

	/***************************************************************************
	 * Updates a category object in the DB
	 * 
	 * @param category
	 *            category to be updated
	 * @throws NonExistingObjectException 
	 * @throws NullObjectException 
	 */
	public void modify(Category category) throws NonExistingObjectException, NullObjectException;

	/***************************************************************************
	 * Removes a category object from the DB
	 * 
	 * @param category
	 *            category to be removed
	 * @throws NonExistingObjectException 
	 * @throws NullObjectException 
	 */
	public void remove(Category category) throws NonExistingObjectException, NullObjectException;

	/***************************************************************************
	 * Returns a collection of all category object stored in the DB.
	 * <p>
	 * If any category isn't found it returns an empty collection.
	 * 
	 * @return a category collection
	 */
	public Collection searchAll();
	
	/***************************************************************************
	 * Returns a collection of all category object stored in the DB.
	 * with the specific language
	 * <p>
	 * If any category isn't found it returns an empty collection.
	 * @param languageID
	 * @return a category collection
	 * @throws NullObjectException 
	 */
	public Collection searchByLanguage(Integer languageID) throws NullObjectException;

}