package com.efetiva.wiiser.server.repository.interfaceBusiness;

import java.util.Collection;



import com.efetiva.wiiser.server.entity.CategoryPlace;
import com.efetiva.wiser.exception.NonExistingObjectException;
import com.efetiva.wiser.exception.NullObjectException;

public interface ICategoryPlaceControl {
	
	/***************************************************************************
	 * Stores a categoryPlace object in the DB
	 * 
	 * @param categoryPlace
	 *            categoryPlace to be stored
	 * @throws NonExistingObjectException 
	 * @throws NullObjectException 
	 */
	public void register(CategoryPlace categoryPlace) throws NonExistingObjectException, NullObjectException;

	/***************************************************************************
	 * Returns a categoryPlace object searched by its identification in the
	 * DB.
	 * <p>
	 * If the categoryPlace isn't found it returns null.
	 * 
	 * @param categoryID, placeID
	 *            the identification in the DB
	 * @return a categoryPlace object
	 * @throws NullObjectException 
	 */
	public CategoryPlace search(Integer categoryID, Integer placeID) throws NullObjectException;

	/***************************************************************************
	 * Removes a categoryPlace object from the DB
	 * 
	 * @param categoryPlace
	 *            categoryPlace to be removed
	 * @throws NonExistingObjectException 
	 * @throws NullObjectException 
	 */
	public void remove(CategoryPlace categoryPlace) throws NonExistingObjectException, NullObjectException;

	/***************************************************************************
	 * Returns a collection of all categoryPlace object stored in the DB.
	 * <p>
	 * If any categoryPlace isn't found it returns an empty collection.
	 * 
	 * @return a categoryPlace collection
	 */
	public Collection searchAll();
	
	/***************************************************************************
	 * Returns a collection of all categoryPlace object stored in the DB
	 * with the specific language ID.
	 * <p>
	 * If any categoryPlace isn't found it returns an empty collection.
	 * 
	 * @param languageID
	 * @return a categoryPlace collection
	 * @throws NullObjectException 
	 */
	public Collection searchByLanguage(Integer languageID) throws NullObjectException;	
	
	/***************************************************************************
	 * Returns a collection of all categoryPlace object stored in the DB
	 * 	with the specific Place ID.
	 * <p>
	 * If any categoryPlace isn't found it returns an empty collection.
	 * 
	 * @return a categoryPlace collection
	 * @throws NullObjectException 
	 */
	public Collection searchByPlace(Integer placeID) throws NullObjectException;
	
	public void removeByPlace(Integer placeID) throws NullObjectException;
}
