package com.efetiva.wiiser.server.repository.interfaceBusiness;

import java.util.Collection;



import com.efetiva.wiiser.server.entity.CategoryService;
import com.efetiva.wiser.exception.ExistingObjectException;
import com.efetiva.wiser.exception.NonExistingObjectException;
import com.efetiva.wiser.exception.NullObjectException;

public interface ICategoryServiceControl {

	/***************************************************************************
	 * Stores a categoryService object in the DB
	 * 
	 * @param categoryService
	 *            categoryService to be stored
	 * @throws ExistingObjectException 
	 * @throws NullObjectException 
	 */
	public void register(CategoryService categoryService) throws ExistingObjectException, NullObjectException;

	/***************************************************************************
	 * Returns a categoryService object searched by its identification in the
	 * DB.
	 * <p>
	 * If the categoryService isn't found it returns null.
	 * 
	 * @param ID_SubCategory, ID_Service
	 *            the identification in the DB
	 * @return a categoryService object
	 * @throws NullObjectException 
	 */
	public CategoryService search(Integer ID_SubCategory, Integer ID_Service) throws NullObjectException;

	/***************************************************************************
	 * Removes a categoryService object from the DB
	 * 
	 * @param categoryService
	 *            categoryService to be removed
	 * @throws NonExistingObjectException 
	 * @throws NullObjectException 
	 */
	public void remove(CategoryService categoryService) throws NonExistingObjectException, NullObjectException;

	/***************************************************************************
	 * Returns a collection of all categoryService object stored in the DB.
	 * <p>
	 * If any categoryService isn't found it returns an empty collection.
	 * 
	 * @return a categoryService collection
	 */
	public Collection searchAll();
	
	/***************************************************************************
	 * Returns a collection of all categoryService object stored in the DB
	 * 		with the specific Service ID.
	 * <p>
	 * If any categoryService isn't found it returns an empty collection.
	 * 
	 * @return a categoryService collection
	 * @throws NullObjectException 
	 */
	public Collection searchByService(Integer ID) throws NullObjectException;
	
	public void removeByService(Integer serviceID) throws NullObjectException;

}