package com.efetiva.wiiser.server.repository.interfaceBusiness;

import java.util.Collection;



import com.efetiva.wiiser.server.entity.DynamicInfo;
import com.efetiva.wiser.exception.ExistingObjectException;
import com.efetiva.wiser.exception.NonExistingObjectException;
import com.efetiva.wiser.exception.NullObjectException;

public interface IDynamicInfoControl {

	/***
	 * Stores a DynamicInfo object in the DB
	 * 
	 * @param	dynamicInfo	DynamicInfo to be stored
	 * @throws ExistingObjectException 
	 * @throws NullObjectException 
	 */
	public void register(DynamicInfo info, Collection categories) throws ExistingObjectException, NullObjectException;

	/***
	 * Returns a DynamicInfo object searched by its
	 * identification in the DB.
	 *  
	 * @param	ID	the identification in the DB
	 * @return		a DynamicInfo object
	 * @throws NullObjectException 
	 */
	public DynamicInfo search(Integer ID) throws NullObjectException;

	/***
	 * Removes a DynamicInfo object from the DB
	 * 
	 * @param	info	DynamicInfo to be removed
	 * @throws NonExistingObjectException 
	 * @throws NullObjectException 
	 */
	public void remove(DynamicInfo info) throws NonExistingObjectException, NullObjectException;

	/***
	 * Updates a DynamicInfo object in the DB
	 * 
	 * @param	info	DynamicInfo to be updated
	 * @throws NonExistingObjectException 
	 * @throws NullObjectException 
	 */
	public void modify(DynamicInfo info, Collection categories) throws NonExistingObjectException, NullObjectException;

	/***
	 * Returns a collection of all DynamicInfo object stored in
	 * the DB.
	 * 
	 * @return	a DynamicInfo collection
	 */
	public Collection searchAll();
	
	/***
	 * Returns a collection of all DynamicInfo object stored in
	 * the DB with the specific languageID.
	 * 
	 * @param languageID
	 * @return	a DynamicInfo collection
	 * @throws NullObjectException 
	 */
	public Collection searchByLanguage(Integer languageID) throws NullObjectException;
	
	public DynamicInfo selectMaxId();

}