package com.efetiva.wiiser.server.repository.interfaceBusiness;

import java.util.Collection;



import com.efetiva.wiiser.server.entity.Place;
import com.efetiva.wiser.exception.ExistingObjectException;
import com.efetiva.wiser.exception.NonExistingObjectException;
import com.efetiva.wiser.exception.NullObjectException;

public interface IPlaceControl {

	/***
	 * Stores a Place object in the DB
	 * 
	 * @param	place	place to be stored
	 * @throws ExistingObjectException 
	 * @throws NullObjectException 
	 * @throws NonExistingObjectException 
	 */
	public void register(Place place, Collection categoryPlace, Collection collPlaceAreas) throws ExistingObjectException, NullObjectException, NonExistingObjectException;

	/***
	 * Returns a Place object searched by its
	 * identification in the DB.
	 *  
	 * @param	ID	the identification in the DB
	 * @return		a Place object
	 * @throws NullObjectException 
	 */
	public Place search(Integer placeID) throws NullObjectException;
	
	/***
	 * Returns a Place object searched by its
	 * name in the DB.
	 *  
	 * @param placeName	 the name in the DB
	 * @return		a Place object
	 * @throws NullObjectException 
	 */
	public Place searchByName(String placeName, Integer languageID) throws NullObjectException;

	/***
	 * Removes a Place object from the DB
	 * 
	 * @param	place	place to be removed
	 * @throws NonExistingObjectException 
	 * @throws NullObjectException 
	 */
	public void remove(Place place) throws NonExistingObjectException, NullObjectException;

	/***
	 * Updates a Place object in the DB
	 * 
	 * @param	place	place to be updated
	 * @throws NonExistingObjectException 
	 * @throws NullObjectException 
	 * @throws ExistingObjectException 
	 */
	public void modify(Place place, Collection categories, Collection collPlaceAreas) throws NonExistingObjectException, NullObjectException, ExistingObjectException;

	/***
	 * Returns a collection of all Place object stored in
	 * the DB.
	 * 
	 * @return	a Place collection
	 */
	public Collection searchAll();
	
	/***
	 * Returns a collection of all Place object stored in
	 * the DB with the specific language.
	 * 
	 * @return	a Place collection
	 * @throws NullObjectException 
	 */
	public Collection searchByLanguage(Integer languageID) throws NullObjectException;

}