package com.efetiva.wiiser.server.repository.interfaceBusiness;

import java.util.Collection;



import com.efetiva.wiiser.server.entity.ScheduleActivity;
import com.efetiva.wiser.exception.NonExistingObjectException;
import com.efetiva.wiser.exception.NullObjectException;

public interface IScheduleActivityControl {

	/***************************************************************************
	 * Stores a ScheduleActivity object in the DB
	 * 
	 * @param schedule
	 *            ScheduleActivity to be stored
	 * @throws NullObjectException 
	 */
	public void register(ScheduleActivity schedule) throws NullObjectException;

	/***************************************************************************
	 * Removes a ScheduleActivity object from the DB
	 * 
	 * @param schedule
	 *            ScheduleActivity to be removed
	 * @throws NullObjectException 
	 */
	public void remove(ScheduleActivity schedule) throws NullObjectException;

	/***************************************************************************
	 * Updates a ScheduleActivity object in the DB
	 * 
	 * @param schedule
	 *            ScheduleActivity to be updated
	 * @throws NonExistingObjectException 
	 * @throws NullObjectException 
	 */
	public void modify(ScheduleActivity schedule) throws NonExistingObjectException, NullObjectException;

	/**
	 * Searches all existing ScheduleActivity
	 * 
	 * @return a ScheduleActivity object collection
	 */
	public Collection searchAll();

	/***************************************************************************
	 * Returns a collection of ScheduleActivity object searched by activity
	 * identification in the DB.
	 * 
	 * @param ID
	 *            activity identification in the DB
	 * @return a ScheduleActivity object
	 * @throws NullObjectException 
	 */
	public Collection searchByActivity(Integer ID) throws NullObjectException;
	
	/***************************************************************************
	 * Returns a collection of ScheduleActivity object searched by activity
	 * name in the DB.
	 * 
	 * @param activityName
	 *            activity name in the DB
	 * @return a ScheduleActivity object
	 * @throws NullObjectException 
	 */
	public Collection searchByActivityName(String activityName, Integer language) throws NullObjectException;
	
	/***************************************************************************
	 * Remove a ScheduleActivity object searched by activity
	 * identifier in the DB.
	 * 
	 * @param ID
	 *            activity identifier in the DB
	 * @return a ScheduleActivity object
	 * @throws NullObjectException 
	 */
	public void removeByActivity(Integer activityID) throws NullObjectException;

	public ScheduleActivity search(Integer scheduleID) throws NullObjectException;
}
