package com.efetiva.wiiser.server.repository.interfaceBusiness;

import java.util.Collection;
import java.util.Date;


import com.efetiva.wiiser.server.entity.Service;
import com.efetiva.wiser.exception.ExistingObjectException;
import com.efetiva.wiser.exception.NonExistingObjectException;
import com.efetiva.wiser.exception.NullObjectException;

public interface IServiceControl {

	/***************************************************************************
	 * Stores a Service object in the DB
	 * 
	 * @param service
	 *            service to be stored
	 * @throws ExistingObjectException
	 * @throws NullObjectException
	 */
	public void register(Service service, Collection categoryService,
			Collection scheduleServices) throws ExistingObjectException,
			NullObjectException;

	/***************************************************************************
	 * Returns a Service object searched by its identification in the DB.
	 * 
	 * @param ID
	 *            the identification in the DB
	 * @return a Service object
	 * @throws NullObjectException
	 */
	public Service search(Integer ID)
			throws NullObjectException;

	/***************************************************************************
	 * Returns a Service object searched by specific date.
	 * 
	 * @param date
	 *            the specific date
	 * @return a Service object collection
	 * @throws NullObjectException
	 */
	public Collection searchServiceByDate(Date date) throws NullObjectException;

	/***************************************************************************
	 * Removes a Service object from the DB
	 * 
	 * @param service
	 *            service to be removed
	 * @throws NonExistingObjectException
	 * @throws NullObjectException
	 */
	public void remove(Service service) throws NonExistingObjectException,
			NullObjectException;

	/***************************************************************************
	 * Updates a Service object in the DB
	 * 
	 * @param service
	 *            service to be updated
	 * @throws NonExistingObjectException
	 * @throws NullObjectException
	 */
	public void modify(Service service, Collection categories,
			Collection schedules) throws NonExistingObjectException,
			NullObjectException, ExistingObjectException;

	/**
	 * Searches all existing services
	 * 
	 * @return a Service object collection
	 */
	public Collection searchAllServices();
	
	/**
	 * Searches all existing services with the specific ID
	 * 
	 * @param languageID
	 * @return a Service object collection
	 * @throws NullObjectException 
	 */
	public Collection searchServicesByLanguage(Integer languageID) throws NullObjectException;

	/**
	 * 
	 * @param placeID
	 * @param type
	 * @return
	 * @throws NullObjectException 
	 */
	public Collection searchServiceByPlace(Integer placeID) throws NullObjectException;

	/**
	 * 
	 * @param rgb
	 * @param languageID
	 * @return
	 * @throws NullObjectException 
	 */
	public Collection searchServiceByArea(Integer rgb, Integer languageID) throws NullObjectException;

	/**
	 * 
	 * @param name
	 * @return
	 * @throws NullObjectException 
	 */
	public Service searchByName(String name, Integer languageID) throws NullObjectException;
}