package com.efetiva.wiiser.server.repository.interfaceRepository;

import java.util.Collection;
import java.util.Date;

import com.efetiva.wiiser.server.entity.Activity;



public interface IActivityRepository {
	
	/***
	 * Stores a Activity object in the DB
	 * 
	 * @param	activity activity to be stored
	 */
	public void register(Activity activity);

	/***
	 * Returns a Activity object searched by its
	 * identification in the DB.
	 *  
	 * @param	ID	the identification in the DB
	 * @return		a Activity object
	 */
	public Activity search(Integer ID);
	
	/***
	 * Returns a Acivity object searched by specific date.
	 *  
	 * @param	date	the specific date
	 * @return			a Activity object collection
	 */
	public Collection searchByDate(Date date, Integer languageID);

	/***
	 * Removes a Activity object from the DB
	 * 
	 * @param	activity	activity to be removed
	 */
	public void remove(Activity activity);

	/***
	 * Updates a Activity object in the DB
	 * 
	 * @param	activity	activity to be updated
	 */
	public void modify(Activity activity);
	
	/**
	 * Searches all existing activities with the specific language
	 * @return	a Activity object collection
	 */
	public Collection searchAll();
	
	/**
	 * Searches all existing activities with the specific language
	 * @return	a Activity object collection
	 */
	public Collection searchByLanguage(Integer languageID);
	
	/**
	 * 
	 * @param placeID
	 * @return
	 */
	public Collection searchByArea(Integer areaID, Integer languageID);
	
	/**
	 * 
	 * @param placeID
	 * @param languageID
	 * @return
	 */
	public Collection searchByPlace(Integer placeID);

	/**
	 * Return an Activity object with the name in parameter
	 * 
	 * @param name
	 * @return Activity
	 */
	public Activity searchByName(String name, Integer languageID);

}
