package com.efetiva.wiiser.server.repository.interfaceRepository;

import java.util.Collection;

import com.efetiva.wiiser.server.entity.CategoryActivity;



public interface ICategoryActivityRepository {

	/***************************************************************************
	 * Stores a categoryActivity object in the DB
	 * 
	 * @param categoryActivity
	 *            categoryActivity to be stored
	 */
	public void register(CategoryActivity categoryActivity);

	/***************************************************************************
	 * Returns a categoryActivity object searched by its identification in the
	 * DB.
	 * <p>
	 * If the categoryActivity isn't found it returns null.
	 * 
	 * @param ID
	 *            the identification in the DB
	 * @return a categoryActivity object
	 */
	public CategoryActivity search(Integer categoryID, Integer activityID);

	/***************************************************************************
	 * Removes a categoryActivity object from the DB
	 * 
	 * @param categoryActivity
	 *            categoryActivity to be removed
	 */
	public void remove(CategoryActivity categoryActivity);

	/***************************************************************************
	 * Returns a collection of all categoryActivity object stored in the DB.
	 * <p>
	 * If any categoryActivity isn't found it returns an empty collection.
	 * 
	 * @return a categoryActivity collection
	 */
	public Collection searchAll();
	
	/***************************************************************************
	 * Returns a collection of all categoryActivity object stored in the DB.
	 * with the specific language
	 * <p>
	 * If any categoryActivity isn't found it returns an empty collection.
	 * 
	 * @param languageID
	 * @return a categoryActivity collection
	 */
	public Collection searchByLanguage(Integer languageID);
	
	/***************************************************************************
	 * Returns a collection of all categoryActivity object stored in the DB with the
	 * 			specific Activity ID.
	 * <p>
	 * If any categoryActivity isn't found it returns an empty collection.
	 * 
	 * @return a categoryActivity collection
	 */
	public Collection searchByActivity(Integer activityID);
	
	
	public void removeByActivity(Integer activityID);
}
