package com.efetiva.wiiser.server.repository.interfaceRepository;

import java.util.Collection;

import com.efetiva.wiiser.server.entity.CategoryPlace;



public interface ICategoryPlaceRepository {

	/***************************************************************************
	 * Stores a categoryPlace object in the DB
	 * 
	 * @param categoryPlace
	 *            categoryPlace to be stored
	 */
	public void register(CategoryPlace categoryPlace);

	/***************************************************************************
	 * Returns a categoryPlace object searched by its identification in the
	 * DB.
	 * <p>
	 * If the categoryPlace isn't found it returns null.
	 * 
	 * @param ID
	 *            the identification in the DB
	 * @return a categoryPlace object
	 */
	public CategoryPlace search(Integer categoryID, Integer placeID);

	/***************************************************************************
	 * Removes a categoryPlace object from the DB
	 * 
	 * @param categoryPlace
	 *            categoryPlace to be removed
	 */
	public void remove(CategoryPlace categoryPlace);

	/***************************************************************************
	 * Returns a collection of all categoryPlace object stored in the DB.
	 * <p>
	 * If any categoryPlace isn't found it returns an empty collection.
	 * 
	 * @return a categoryPlace collection
	 */
	public Collection searchAll();
	
	/***************************************************************************
	 * Returns a collection of all categoryPlace object stored in the DB
	 * with the specific language.
	 * <p>
	 * If any categoryPlace isn't found it returns an empty collection.
	 * 
	 * @return a categoryPlace collection
	 */
	public Collection searchByLanguage(Integer languageID);
	
	/***************************************************************************
	 * Returns a collection of all categoryPlace object stored in the DB
	 * 		with the specific Place ID.
	 * <p>
	 * If any categoryPlace isn't found it returns an empty collection.
	 * 
	 * @return a categoryPlace collection
	 */
	public Collection searchByPlace(Integer placeID);

	public void removeByPlace(Integer placeID);
}
