package com.efetiva.wiiser.server.repository.interfaceRepository;

import java.util.Collection;

import com.efetiva.wiiser.server.entity.Category;



public interface ICategoryRepository {

	/***************************************************************************
	 * Stores a category object in the DB
	 * 
	 * @param category
	 *            category to be stored
	 */
	public void register(Category category);

	/***************************************************************************
	 * Returns a category object searched by its identification in the DB.
	 * <p>
	 * If the category isn't found it returns null.
	 * 
	 * @param ID
	 *            the identification in the DB
	 * @return a category object
	 */
	public Category search(Integer ID);

	/***************************************************************************
	 * Removes a category object from the DB
	 * 
	 * @param category
	 *            category to be removed
	 */
	public void remove(Category category);

	/***************************************************************************
	 * Updates a category object in the DB
	 * 
	 * @param category
	 *            category to be updated
	 */
	public void modify(Category category);

	/***************************************************************************
	 * Returns a collection of all category object stored in the DB.
	 * <p>
	 * If any category isn't found it returns an empty collection.
	 * 
	 * @return a category collection
	 */
	public Collection searchAll();
	
	/***************************************************************************
	 * Returns a collection of all category object stored in the DB
	 * with the specific language.
	 * <p>
	 * If any category isn't found it returns an empty collection.
	 * 
	 * @return a category collection
	 */
	public Collection searchByLanguage(Integer languageID);

}