package com.efetiva.wiiser.server.repository.interfaceRepository;

import java.util.Collection;

import com.efetiva.wiiser.server.entity.PlaceArea;


public interface IPlaceAreaRepository {

	/***************************************************************************
	 * Stores a Place object in the DB
	 * 
	 * @param place
	 *            place to be stored
	 */
	public abstract void register(PlaceArea placeArea);

	/**
	 * 
	 * @param id
	 * @return
	 */
	public abstract PlaceArea search(Integer id);
	
	/***************************************************************************
	 * Returns a PlaceArea object searched by its identification in the DB.
	 * <p>
	 * If the PlaceArea isn't found it returns null.
	 * 
	 * @param ID
	 *            the identification in the DB
	 * @return a PlaceArea object
	 */
	public abstract PlaceArea searchByPlaceArea(Integer placeID, Integer areaID);

	/***************************************************************************
	 * Removes a PlaceArea object from the DB
	 * 
	 * @param placeArea
	 *            placeArea to be removed
	 */
	public abstract void remove(PlaceArea placeArea);

	/***************************************************************************
	 * Updates a PlaceArea object in the DB
	 * 
	 * @param placeArea
	 *            placeArea to be updated
	 */
	public abstract void modify(PlaceArea placeArea);

	/***************************************************************************
	 * Returns a collection of all PlaceArea object stored in the DB.
	 * <p>
	 * If any PlaceArea isn't found it returns an empty collection.
	 * 
	 * @return a PlaceArea collection
	 */
	public abstract Collection searchAll();

	public abstract Collection searchByArea(Integer areaID, Integer languageID);

	public abstract Collection searchByPlace(Integer placeID);
	
	public void removeByPlace(Integer placeID);

}