package com.efetiva.wiiser.server.repository.interfaceRepository;

import java.util.Collection;
import java.util.Date;

import com.efetiva.wiiser.server.entity.Service;




public interface IServiceRepository {

	/***
	 * Stores a Service object in the DB
	 * 
	 * @param	service	service to be stored
	 */
	public void register(Service service);

	/***
	 * Returns a Service object searched by its
	 * identification in the DB.
	 *  
	 * @param	ID	the identification in the DB
	 * @return		a Service object
	 */
	public Service search(Integer ID);
	
	/***
	 * Returns a Service object searched by specific date.
	 *  
	 * @param	date	the specific date
	 * @return			a Service object collection
	 */
	public Collection searchServiceByDate(Date date);

	/***
	 * Removes a Service object from the DB
	 * 
	 * @param	service	service to be removed
	 */
	public void remove(Service service);

	/***
	 * Updates a Service object in the DB
	 * 
	 * @param	service	service to be updated
	 */
	public void modify(Service service);
	
	/**
	 * Searches all existing services
	 * @return	a Service object collection
	 */
	public Collection searchAllServices();
	
	/**
	 * Searches all existing services with the specific anguageID
	 * 
	 * @param languageID
	 * @return	a Service object collection
	 */
	public Collection searchServicesByLanguage(Integer languageID);
	
	/**
	 * 
	 * @param placeID
	 * @return
	 */
	public Collection searchServiceByPlace(Integer placeID);
	
	/**
	 * 
	 * @param rgb
	 * @param languageID
	 * @return
	 */
	public Collection searchServiceByArea(Integer rgb,
			Integer languageID);
	
	/**
	 * 
	 * @param name
	 * @return
	 */
	public Service searchByName(String name, Integer languageID);
}