package com.efetiva.wiiser.server.singleton;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;

import org.hibernate.HibernateException;
import org.hibernate.Session;
import org.hibernate.Transaction;

import com.efetiva.wiiser.server.business.ActivityControl;
import com.efetiva.wiiser.server.business.AreaControl;
import com.efetiva.wiiser.server.business.CategoryActivityControl;
import com.efetiva.wiiser.server.business.CategoryControl;
import com.efetiva.wiiser.server.business.CategoryPlaceControl;
import com.efetiva.wiiser.server.business.CategoryServiceControl;
import com.efetiva.wiiser.server.business.DynamicInfoControl;
import com.efetiva.wiiser.server.business.LanguageControl;
import com.efetiva.wiiser.server.business.MapControl;
import com.efetiva.wiiser.server.business.PlaceAreaControl;
import com.efetiva.wiiser.server.business.PlaceControl;
import com.efetiva.wiiser.server.business.ScheduleActivityControl;
import com.efetiva.wiiser.server.business.ScheduleInfoControl;
import com.efetiva.wiiser.server.business.ScheduleServiceControl;
import com.efetiva.wiiser.server.business.ServiceControl;
import com.efetiva.wiiser.server.business.UserControl;
import com.efetiva.wiiser.server.entity.Activity;
import com.efetiva.wiiser.server.entity.Area;
import com.efetiva.wiiser.server.entity.Category;
import com.efetiva.wiiser.server.entity.CategoryActivity;
import com.efetiva.wiiser.server.entity.CategoryPlace;
import com.efetiva.wiiser.server.entity.CategoryService;
import com.efetiva.wiiser.server.entity.DynamicInfo;
import com.efetiva.wiiser.server.entity.Language;
import com.efetiva.wiiser.server.entity.Map;
import com.efetiva.wiiser.server.entity.Place;
import com.efetiva.wiiser.server.entity.PlaceArea;
import com.efetiva.wiiser.server.entity.ScheduleActivity;
import com.efetiva.wiiser.server.entity.ScheduleInfo;
import com.efetiva.wiiser.server.entity.ScheduleService;
import com.efetiva.wiiser.server.entity.Service;
import com.efetiva.wiiser.server.entity.User;
import com.efetiva.wiiser.server.repository.interfaceBusiness.IActivityControl;
import com.efetiva.wiiser.server.repository.interfaceBusiness.IAreaControl;
import com.efetiva.wiiser.server.repository.interfaceBusiness.ICategoryActivityControl;
import com.efetiva.wiiser.server.repository.interfaceBusiness.ICategoryControl;
import com.efetiva.wiiser.server.repository.interfaceBusiness.ICategoryPlaceControl;
import com.efetiva.wiiser.server.repository.interfaceBusiness.ICategoryServiceControl;
import com.efetiva.wiiser.server.repository.interfaceBusiness.IDynamicInfoControl;
import com.efetiva.wiiser.server.repository.interfaceBusiness.ILanguageControl;
import com.efetiva.wiiser.server.repository.interfaceBusiness.IMapControl;
import com.efetiva.wiiser.server.repository.interfaceBusiness.IPlaceAreaControl;
import com.efetiva.wiiser.server.repository.interfaceBusiness.IPlaceControl;
import com.efetiva.wiiser.server.repository.interfaceBusiness.IScheduleActivityControl;
import com.efetiva.wiiser.server.repository.interfaceBusiness.IScheduleInfoControl;
import com.efetiva.wiiser.server.repository.interfaceBusiness.IScheduleServiceControl;
import com.efetiva.wiiser.server.repository.interfaceBusiness.IServiceControl;
import com.efetiva.wiiser.server.repository.interfaceBusiness.IUserControl;
import com.efetiva.wiiser.server.util.hibernate.HibernateUtil;
import com.efetiva.wiser.exception.ExistingObjectException;
import com.efetiva.wiser.exception.NonExistingObjectException;
import com.efetiva.wiser.exception.NullObjectException;

public class Facade {

	private IActivityControl activities;

	private IAreaControl areas;

	private IDynamicInfoControl dynamicInfos;

	private IPlaceControl places;

	private IMapControl map;

	private IServiceControl services;

	private ILanguageControl languages;

	private ICategoryControl categories;

	private IUserControl users;

	private IScheduleInfoControl scheduleInfos;

	private IScheduleServiceControl scheduleServices;

	private IScheduleActivityControl scheduleActivities;

	private ICategoryServiceControl categoryServices;

	private ICategoryPlaceControl categoryPlaces;

	private ICategoryActivityControl categoryActivities;

	private IPlaceAreaControl placeAreas;

	private static Facade SINGLE_INSTANCE = null;

	/***************************************************************************
	 * Returns the unique Facade object instance.
	 * <p>
	 * If the SINGLE_INSTANCE field is null, it's because there isn't a Facade
	 * object. So it creates and returns a new Facade.
	 * 
	 * @return the SINGLE_INSTANCE field
	 */
	public static Facade getInstance() {
		if (SINGLE_INSTANCE == null) {
			SINGLE_INSTANCE = new Facade();
		}
		return SINGLE_INSTANCE;
	}

	/***************************************************************************
	 * Constructor of Facade
	 * 
	 */
	private Facade() {
		activities = new ActivityControl();
		areas = new AreaControl();
		dynamicInfos = new DynamicInfoControl();
		places = new PlaceControl();
		map = new MapControl();
		services = new ServiceControl();
		categories = new CategoryControl();
		categories = new CategoryControl();
		users = new UserControl();
		scheduleInfos = new ScheduleInfoControl();
		scheduleServices = new ScheduleServiceControl();
		scheduleActivities = new ScheduleActivityControl();
		categoryServices = new CategoryServiceControl();
		categoryPlaces = new CategoryPlaceControl();
		categoryActivities = new CategoryActivityControl();
		languages = new LanguageControl();
		placeAreas = new PlaceAreaControl();
	}

	/*
	 * *****************************AREA*********************************
	 */

	/***************************************************************************
	 * Stores an Area object in the DB
	 * 
	 * @param area
	 *            area to be stored
	 * @throws NullObjectException
	 * @throws ExistingObjectException
	 */
	public void registerArea(Area area) throws ExistingObjectException,
	NullObjectException {


		this.areas.register(area);
	}

	/***************************************************************************
	 * Returns an Area object searched by its identification in the DB.
	 * <p>
	 * If the Area isn't found it returns null.
	 * 
	 * @param ID
	 *            the identification in the DB
	 * @return an Area object
	 * @throws NullObjectException 
	 */
	public Area searchArea(Integer ID) throws NullObjectException {


		Area area = null;
		area = this.areas.search(ID);
		return area;
	}

	/***************************************************************************
	 * Returns an Area object searched by its identification in the DB.
	 * <p>
	 * If the Area isn't found it returns null.
	 * 
	 * @param ID
	 *            the identification in the DB
	 * @return an Area object
	 * @throws NullObjectException 
	 */
	public Area searchAreaByName(String name) throws NullObjectException {
		Area area = null;
		area = this.areas.searchByName(name);
		return area;
	}

	/***************************************************************************
	 * Removes an Area object from the DB
	 * 
	 * @param area
	 *            area to be removed
	 * @throws NullObjectException
	 * @throws NonExistingObjectException
	 */
	public void removeArea(Area area) throws NonExistingObjectException,
	NullObjectException {
		this.areas.remove(area);
	}

	/***************************************************************************
	 * Updates an Area object in the DB
	 * 
	 * @param area
	 *            area to be updated
	 * @throws NullObjectException
	 * @throws NonExistingObjectException
	 */
	public void modifyArea(Area area) throws NonExistingObjectException,
	NullObjectException {
		this.areas.modify(area);
	}

	/***************************************************************************
	 * Returns a collection of all Area object stored in the DB.
	 * <p>
	 * If any Area isn't found it returns an empty collection.
	 * 
	 * @return an Area collection
	 */
	public Collection searchAllAreas() {
		Collection areas = new ArrayList();
		areas = this.areas.searchAll();
		return areas;
	}

	/***************************************************************************
	 * Returns a collection of all Area object stored in the DB.
	 * <p>
	 * If any Area isn't found it returns an empty collection.
	 * 
	 * @return an Area collection
	 */
	public Collection searchAreasByLanguage(Integer languageID) {


		Collection areas = new ArrayList();
		try {
			areas = this.areas.searchByLanguage(languageID);
		} catch (NullObjectException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return areas;
	}

	/*
	 * *****************************DYNAMICINFO**************************
	 */

	/***************************************************************************
	 * Stores a DynamicInfo object in the DB
	 * 
	 * @param info
	 *            DynamicInfo to be stored
	 * @throws NullObjectException
	 * @throws ExistingObjectException
	 */
	public void registerDynamicInfo(DynamicInfo info, Collection schedulesInfo)
	throws ExistingObjectException, NullObjectException {
		this.dynamicInfos.register(info, schedulesInfo);
	}

	/***************************************************************************
	 * Returns a DynamicInfo object searched by its identification in the DB.
	 * <p>
	 * If the DynamicInfo isn't found it returns null.
	 * 
	 * @param ID
	 *            the identification in the DB
	 * @return a DynamicInfo object
	 * @throws NullObjectException
	 */
	public DynamicInfo searchDynamicInfo(Integer ID) throws NullObjectException {


		DynamicInfo info = null;
		info = this.dynamicInfos.search(ID);
		return info;
	}

	/***************************************************************************
	 * Removes a DynamicInfo object from the DB
	 * 
	 * @param info
	 *            DynamicInfo to be removed
	 * @throws NullObjectException
	 * @throws NonExistingObjectException
	 */
	public void removeDynamicInfo(DynamicInfo info)
	throws NonExistingObjectException, NullObjectException {
		this.dynamicInfos.remove(info);
	}

	/***************************************************************************
	 * Updates a DynamicInfo object in the DB
	 * 
	 * @param info
	 *            DynamicInfo to be updated
	 * @throws NullObjectException
	 * @throws NonExistingObjectException
	 */
	public void modifyDynamicInfo(DynamicInfo info, Collection categories)
	throws NonExistingObjectException, NullObjectException {
		this.dynamicInfos.modify(info, categories);
	}

	/***************************************************************************
	 * Returns a collection of all DynamicInfo object stored in the DB.
	 * <p>
	 * If any DynamicInfo isn't found it returns an empty collection.
	 * 
	 * @return a DynamicInfo collection
	 */
	public Collection searchAllDynamicInfo() {
		Collection infos = new ArrayList();
		infos = this.dynamicInfos.searchAll();
		return infos;
	}

	/***************************************************************************
	 * Returns a collection of all DynamicInfo object stored in the DB
	 * with the specific languageID.
	 * <p>
	 * If any DynamicInfo isn't found it returns an empty collection.
	 * 
	 * @return a DynamicInfo collection
	 * @throws NullObjectException 
	 */
	public Collection searchDynamicInfoByLanguage(Integer languageID) throws NullObjectException {
		Collection infos = new ArrayList();
		infos = this.dynamicInfos.searchByLanguage(languageID);
		return infos;
	}

	public DynamicInfo selectMaxId(){
		return this.dynamicInfos.selectMaxId();
	}

	/*
	 * *****************************PLACE**************************
	 */

	/***************************************************************************
	 * Stores a Place object in the DB
	 * 
	 * @param place
	 *            place to be stored
	 * @throws NullObjectException
	 * @throws ExistingObjectException
	 * @throws NonExistingObjectException
	 */
	public void registerPlace(Place place, Collection categoryPlace,
			Collection placeAreas) throws ExistingObjectException,
			NullObjectException, NonExistingObjectException {
		this.places.register(place, categoryPlace, placeAreas);
	}

	/***************************************************************************
	 * Returns a Place object searched by its identification in the DB.
	 * <p>
	 * If the Place isn't found it returns null.
	 * 
	 * @param ID
	 *            the identification in the DB
	 * @return a Place object
	 * @throws NullObjectException
	 */
	public Place searchPlace(Integer placeID)
	throws NullObjectException {


		Place place = null;
		place = this.places.search(placeID);
		return place;
	}

	/***************************************************************************
	 * Returns a Place object searched by its name in the DB.
	 * <p>
	 * If the Place isn't found it returns null.
	 * 
	 * @param ID
	 *            the identification in the DB
	 * @return a Place object
	 * @throws NullObjectException
	 */
	public Place searchPlaceByName(String placeName, Integer languageID)
	throws NullObjectException {


		Place place = null;
		place = this.places.searchByName(placeName, languageID);
		return place;
	}

	/***************************************************************************
	 * Removes a Place object from the DB
	 * 
	 * @param place
	 *            place to be removed
	 * @throws NullObjectException
	 * @throws NonExistingObjectException
	 */
	public void removePlace(Place place) throws NonExistingObjectException,
	NullObjectException {
		this.places.remove(place);
	}

	/***************************************************************************
	 * Updates a Place object in the DB
	 * 
	 * @param place
	 *            place to be updated
	 * @throws NullObjectException
	 * @throws NonExistingObjectException
	 * @throws ExistingObjectException
	 */
	public void modifyPlace(Place place, Collection categories,
			Collection placeAreas) throws NonExistingObjectException,
			NullObjectException, ExistingObjectException {
		this.places.modify(place, categories, placeAreas);
	}

	/***************************************************************************
	 * Returns a collection of all Place object stored in the DB.
	 * <p>
	 * If any Place isn't found it returns an empty collection.
	 * 
	 * @return a Place collection
	 */
	public Collection searchAllPlaces() {
		Collection places = new ArrayList();
		places = this.places.searchAll();
		return places;
	}

	/***************************************************************************
	 * Returns a collection of all Place object stored in the DB
	 * with the specific language.
	 * <p>
	 * If any Place isn't found it returns an empty collection.
	 * 
	 * @param languageID
	 * @return a Place collection
	 * @throws NullObjectException 
	 */
	public Collection searchPlacesByLanguage(Integer languageID) throws NullObjectException {
		Collection places = new ArrayList();
		places = this.places.searchByLanguage(languageID);
		return places;
	}

	/*
	 * *********************************MAP******************************
	 */

	/***************************************************************************
	 * Stores a Map object in the DB
	 * 
	 * @param map
	 *            map to be stored
	 * @throws ExistingObjectException
	 */
	public void registerMap(Map map) throws ExistingObjectException {
		try {
			this.map.register(map);
		} catch (NullObjectException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	/***************************************************************************
	 * Returns a Map object searched by its identification in the DB.
	 * <p>
	 * If the Map isn't found it returns null.
	 * 
	 * @param ID
	 *            the identification in the DB
	 * @return a Map object
	 */
	public Map searchMap(Integer ID) {
		Map map = null;
		try {
			map = this.map.search(ID);
		} catch (NullObjectException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return map;
	}

	/***************************************************************************
	 * Updates a Map object in the DB
	 * 
	 * @param map
	 *            map to be updated
	 * @throws NonExistingObjectException
	 */
	public void modifyMap(Map map) throws NonExistingObjectException {
		try {
			this.map.modify(map);
		} catch (NullObjectException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	/*
	 * *****************************SERVICE******************************
	 */

	/***************************************************************************
	 * Stores a Service object in the DB
	 * 
	 * @param service
	 *            service to be stored
	 * @throws NullObjectException
	 * @throws ExistingObjectException
	 */
	public void registerService(Service service, Collection categoryService,
			Collection scheduleServices) throws ExistingObjectException,
			NullObjectException {
		this.services.register(service, categoryService, scheduleServices);
	}

	/***************************************************************************
	 * Returns a Service object searched by its identification in the DB.
	 * <p>
	 * If the Service isn't found it returns null.
	 * 
	 * @param ID
	 *            the identification in the DB
	 * @return a Service object
	 * @throws NullObjectException
	 */
	public Service searchService(Integer ID)
	throws NullObjectException {
		Service service = null;
		service = this.services.search(ID);
		return service;
	}

	public Collection searchServiceByArea(Integer rgb, Integer languageID) {


		Collection allServices = new ArrayList();
		try {
			allServices = this.services.searchServiceByArea(rgb, languageID);
		} catch (NullObjectException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return allServices;
	}

	public Service searchServiceByName(String name, Integer languageID) {


		Service service =null;
		try {
			service = this.services.searchByName(name, languageID);
		} catch (NullObjectException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return service;
	}
	/***************************************************************************
	 * Returns a Service object searched by specific date.
	 * <p>
	 * If any Service isn't found it returns an empty collection.
	 * 
	 * @param date
	 *            the specific date
	 * @return a Service object collection
	 * @throws NullObjectException
	 */
	public Collection searchServiceByDate(Date date) throws NullObjectException {


		Collection services = new ArrayList();
		services = this.services.searchServiceByDate(date);
		return services; 
	}

	/***************************************************************************
	 * Removes a Service object from the DB
	 * 
	 * @param service
	 *            service to be removed
	 * @throws NullObjectException
	 * @throws NonExistingObjectException
	 */
	public void removeService(Service service)
	throws NonExistingObjectException, NullObjectException {

		this.services.remove(service);
	}

	/***************************************************************************
	 * Updates a Service object in the DB
	 * 
	 * @param service
	 *            service to be updated
	 * @throws NonExistingObjectException
	 * @throws NullObjectException
	 */
	public void modifyService(Service service, Collection categories,
			Collection schedules) throws NonExistingObjectException,
			NullObjectException, ExistingObjectException {
		this.services.modify(service, categories, schedules);
	}

	/**
	 * Searches all existing services
	 * 
	 * @return a Service object collection
	 */
	public Collection searchAllServices() {


		Collection allServices = new ArrayList();
		allServices = this.services.searchAllServices();
		return allServices;
	}

	/**
	 * Searches all existing services with the specific languageID
	 * 
	 * @param languageID 
	 * @return a Service object collection
	 */
	public Collection searchServicesByLanguage(Integer languageID) {


		Collection allServices = new ArrayList();
		try {
			allServices = this.services.searchServicesByLanguage(languageID);
		} catch (NullObjectException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return allServices;
	}

	public Collection searchServiceByPlace(Integer placeID) {


		Collection allServices = new ArrayList();
		try {
			allServices = this.services.searchServiceByPlace(placeID);
		} catch (NullObjectException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return allServices;
	}

	/*
	 * *****************************USER*********************************
	 */

	/***************************************************************************
	 * Stores a User object in the DB
	 * 
	 * @param user
	 *            user to be stored
	 * @throws NullObjectException
	 */
	public void registerUser(User user) throws NullObjectException {
		this.users.register(user);
	}

	/***************************************************************************
	 * Returns a User object searched by its username (identification in the
	 * DB).
	 * <p>
	 * If the User isn't found it returns null.
	 * 
	 * @param username
	 *            username that is the identification in the DB
	 * @return a User object
	 */
	public User searchUser(String username) {


		User user = null;
		try {
			user = this.users.search(username);
		} catch (NullObjectException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return user;
	}

	/***************************************************************************
	 * Removes a User object from the DB
	 * 
	 * @param user
	 *            user to be removed
	 * @throws NullObjectException
	 */
	public void removeUser(User user) throws NullObjectException {
		this.users.remove(user);
	}

	/***************************************************************************
	 * Updates a User object in the DB
	 * 
	 * @param user
	 *            user to be updated
	 * @throws NonExistingObjectException
	 */
	public void modifyUser(User user) throws NonExistingObjectException {
		try {
			this.users.modify(user);
		} catch (NullObjectException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public Collection searchAllUsers() {
		Collection allUsers = new ArrayList();
		allUsers = this.users.searchAll();
		return allUsers;
	}

	/*
	 * *****************************ScheduleInfo*********************************
	 */

	/***************************************************************************
	 * Stores a ScheduleInfo object in the DB
	 * 
	 * @param scheduleInfo
	 *            ScheduleInfo to be stored
	 */
	public void registerScheduleInfo(ScheduleInfo scheduleInfo) {
		try {
			this.scheduleInfos.register(scheduleInfo);
		} catch (NullObjectException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	/***************************************************************************
	 * Removes a ScheduleInfo object from the DB
	 * 
	 * @param scheduleInfo
	 *            ScheduleInfo to be removed
	 */
	public void removeScheduleInfo(ScheduleInfo scheduleInfo) {
		try {
			this.scheduleInfos.remove(scheduleInfo);
		} catch (NullObjectException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	/***************************************************************************
	 * Updates a ScheduleInfo object in the DB
	 * 
	 * @param scheduleInfo
	 *            ScheduleInfo to be updated
	 */
	public void modifyScheduleInfo(ScheduleInfo scheduleInfo) {
		try {
			this.scheduleInfos.modify(scheduleInfo);
		} catch (NullObjectException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public Collection searchAllScheduleInfo() {


		Collection schedules = new ArrayList();
		schedules = this.scheduleInfos.searchAll();
		return schedules;
	}

	public Collection searchScheduleInfoByDynamicInfo(Integer id) {


		Collection schedules = new ArrayList();
		try {
			schedules = this.scheduleInfos.searchByDynamicInfo(id);
		} catch (NullObjectException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return schedules;
	}

	public Collection searchScheduleInfoByHour(Date date) {


		Collection schedules = new ArrayList();
		try {
			schedules = this.scheduleInfos.searchByHour(date);
		} catch (NullObjectException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return schedules;
	}

	/*
	 * *****************************SCHEDULESERVICE*********************************
	 */

	/***************************************************************************
	 * Stores a ScheduleService object in the DB
	 * 
	 * @param scheduleService
	 *            ScheduleService to be stored
	 */
	public void registerScheduleService(ScheduleService scheduleService) {
		try {
			this.scheduleServices.register(scheduleService);
		} catch (NullObjectException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	/***************************************************************************
	 * Removes a ScheduleService object from the DB
	 * 
	 * @param scheduleService
	 *            ScheduleService to be removed
	 */
	public void removeScheduleService(ScheduleService scheduleService) {
		try {
			this.scheduleServices.remove(scheduleService);
		} catch (NullObjectException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	/***************************************************************************
	 * Updates a ScheduleService object in the DB
	 * 
	 * @param scheduleService
	 *            ScheduleService to be updated
	 */
	public void modifyScheduleService(ScheduleService scheduleService) {
		try {
			this.scheduleServices.modify(scheduleService);
		} catch (NullObjectException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	/***************************************************************************
	 * Updates a ScheduleService object in the DB
	 * 
	 * @param scheduleService
	 *            ScheduleService to be updated
	 */
	public ScheduleService searchScheduleService(Integer id) {


		ScheduleService schedule = null;
		try {
			schedule = this.scheduleServices.search(id);
		} catch (NullObjectException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return schedule;
	}

	/***************************************************************************
	 * Returns a collection of ScheduleService object searched by service
	 * identification in the DB.
	 * <p>
	 * If any ScheduleService isn't found it returns an empty collection.
	 * 
	 * @param ID
	 *            service identification in the DB
	 * @return a ScheduleService object
	 */
	public Collection searchScheduleByService(Integer ID) {


		Collection schedules = new ArrayList();
		try {
			schedules = this.scheduleServices.searchByService(ID);
		} catch (NullObjectException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return schedules; 
	}

	public Collection searchAllScheduleService() {


		Collection schedules = new ArrayList();
		schedules = this.scheduleServices.searchAll();
		return schedules;
	}

	/*
	 * *****************************ScheduleActivity*********************************
	 */

	/**
	 * Stores a ScheduleActivity object in the DB
	 * 
	 * @param scheduleActivity
	 *            scheduleActivity to be stored
	 * @throws NullObjectException
	 */
	public void registerScheduleActivity(ScheduleActivity scheduleActivity)
	throws NullObjectException {
		this.scheduleActivities.register(scheduleActivity);
	}

	/**
	 * Removes a ScheduleActivity object from the DB
	 * 
	 * @param scheduleActivity
	 *            ScheduleActivity to be removed
	 * @throws NullObjectException
	 */
	public void removeScheduleActivity(ScheduleActivity scheduleActivity)
	throws NullObjectException {
		this.scheduleActivities.remove(scheduleActivity);
	}

	/**
	 * Updates a ScheduleActivity object in the DB
	 * 
	 * @param scheduleActivity
	 *            ScheduleActivity to be updated
	 * @throws NonExistingObjectException
	 */
	public void modifyScheduleActivity(ScheduleActivity scheduleActivity)
	throws NonExistingObjectException {
		try {
			this.scheduleActivities.modify(scheduleActivity);
		} catch (NullObjectException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public ScheduleActivity searchScheduleActivity(Integer scheduleID) {


		ScheduleActivity schedule = null;
		try {
			schedule = this.scheduleActivities.search(scheduleID);
		} catch (NullObjectException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return schedule;
	}

	/***************************************************************************
	 * Returns a collection of ScheduleActivity object searched by Activity
	 * identification in the DB.
	 * <p>
	 * If any ScheduleActivity isn't found it returns an empty collection.
	 * 
	 * @param ID
	 *            Activity identification in the DB
	 * @return a ScheduleActivity object
	 */
	public Collection searchScheduleByActivity(Integer activityID) {


		Collection schedules = new ArrayList();
		try {
			schedules = this.scheduleActivities.searchByActivity(activityID);
		} catch (NullObjectException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return schedules;
	}

	/***************************************************************************
	 * Returns a collection of ScheduleActivity object searched by Activity
	 * name in the DB.
	 * <p>
	 * If any ScheduleActivity isn't found it returns an empty collection.
	 * 
	 * @param ID
	 *            Activity name in the DB
	 * @return a ScheduleActivity object
	 */
	public Collection searchScheduleByActivityName(String activityName,
			Integer languageID) {


		Collection schedules = new ArrayList();
		try {
			schedules = this.scheduleActivities.searchByActivityName(activityName, languageID);
		} catch (NullObjectException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return schedules;
	}

	public Collection searchAllScheduleActivity() {


		Collection schedules = new ArrayList();
		schedules = this.scheduleActivities.searchAll();
		return schedules;
	}

	/*
	 * ******************************ACTIVITY**************************************
	 * 
	 */

	/**
	 * 
	 * @param activity
	 * @throws NonExistingObjectException
	 * @throws NullObjectException
	 * @throws ExistingObjectException
	 */
	public void modifyActivity(Activity activity, Collection categories,
			Collection schedules) throws NonExistingObjectException,
			ExistingObjectException, NullObjectException {
		this.activities.modify(activity, categories, schedules);
	}

	/**
	 * 
	 * @param activity
	 * @throws ExistingObjectException
	 * @throws NullObjectException
	 */
	public void registerActivity(Activity activity,
			Collection categoryActivity, Collection scheduleActivities)
	throws ExistingObjectException, NullObjectException {
		this.activities.register(activity, categoryActivity, scheduleActivities);
	}

	/**
	 * 
	 * @param activity
	 * @throws NonExistingObjectException
	 * @throws NullObjectException
	 */
	public void removeActivity(Activity activity)
	throws NonExistingObjectException, NullObjectException {
		this.activities.remove(activity);
	}

	/**
	 * 
	 * @param ID
	 * @return
	 * @throws NonExistingObjectException
	 * @throws NullObjectException 
	 */
	public Activity searchActivity(Integer ID)
	throws NonExistingObjectException, NullObjectException {


		Activity activity = null;
		activity = this.activities.search(ID);
		return activity;
	}

	/**
	 * 
	 * @param date
	 * @return
	 */
	public Collection searchActivityByDate(Date date, Integer languageID) {


		Collection activities = new ArrayList();
		try {
			activities = this.activities.searchByDate(date, languageID);
		} catch (NullObjectException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return activities;
	}

	/**
	 * 
	 * @param placeID
	 * @return
	 */
	public Collection searchActivityByArea(Integer areaID, Integer languageID) {


		Collection activities = new ArrayList();
		try {
			activities = this.activities.searchByArea(areaID, languageID);
		} catch (NullObjectException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return activities;
	}

	/**
	 * Return an Activity object with the name in parameter
	 * 
	 * @param name
	 * @return Activity
	 */
	public Activity searchActivityByName(String name, Integer languageID){


		Activity activity = null;
		try {
			activity = this.activities.searchByName(name, languageID);
		} catch (NullObjectException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return activity;

	}

	/**
	 * 
	 * @param placeID
	 * @return
	 */
	public Collection searchActivityByPlace(Integer placeID) {


		Collection activities = new ArrayList();
		try {
			activities = this.activities.searchByPlace(placeID);
		} catch (NullObjectException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return activities;
	}

	/**
	 * 
	 * @return
	 */
	public Collection searchAllActivities() {


		Collection activities = this.activities.searchAll();
		return activities;
	}

	/**
	 * 
	 * @return
	 */
	public Collection searchActivitiesByLanguage(Integer languageID) {


		Collection activities = new ArrayList();
		try {
			activities = this.activities.searchByLanguage(languageID);
		} catch (NullObjectException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return activities;
	}

	/*
	 * ***************************LANGUAGE**********************************
	 */

	/**
	 * @param language
	 *            The language that will be modified
	 * @throws NullObjectException
	 * @throws NonExistingObjectException
	 */
	public void modifyLanguage(Language language)
	throws NonExistingObjectException, NullObjectException {
		this.languages.modify(language);

	}

	/**
	 * @param language
	 *            The language that will be register
	 * @throws NullObjectException
	 * @throws ExistingObjectException
	 */
	public void registerLanguage(Language language)
	throws ExistingObjectException, NullObjectException {
		this.languages.register(language);
	}

	/**
	 * @param language
	 *            The language that will be removed
	 * @throws NullObjectException
	 * @throws NonExistingObjectException
	 */
	public void removeLanguage(Language language)
	throws NonExistingObjectException, NullObjectException {
		this.languages.remove(language);
	}

	/**
	 * @param ID
	 *            language ID for the search in Data base
	 * @return Language the language with the especific ID null if language dos
	 *         not exists
	 * @throws NullObjectException
	 */
	public Language searchLanguage(Integer ID) throws NullObjectException {


		Language language = this.languages.search(ID);
		return language;
	}

	/**
	 * @param ID
	 *            language ID for the search in Data base
	 * @return Language the language with the especific ID null if language dos
	 *         not exists
	 * @throws NullObjectException
	 */
	public Language searchLanguageByName(String name) throws NullObjectException {


		Language language = this.languages.searchByName(name);
		return language;
	}

	/**
	 * @return Collection the collection of all existing languages
	 */
	public Collection searchAllLanguages() {


		Collection languages = new ArrayList();
		languages = this.languages.searchAll();
		return languages;
	}

	/*
	 * ***************************CATEGORY**********************************
	 */

	/**
	 * @param category
	 *            The category that will be modified
	 * @throws NullObjectException
	 * @throws NonExistingObjectException
	 */
	public void modifyCategory(Category category)
	throws NonExistingObjectException, NullObjectException {
		this.categories.modify(category);
	}

	/**
	 * @param category
	 *            The category that will be register
	 * @throws NullObjectException
	 * @throws ExistingObjectException
	 */
	public void registerCategory(Category category)
	throws ExistingObjectException, NullObjectException {
		this.categories.register(category);
	}

	/**
	 * @param category
	 *            The category that will be removed
	 * @throws NullObjectException
	 * @throws NonExistingObjectException
	 */
	public void removeCategory(Category category)
	throws NonExistingObjectException, NullObjectException {

		this.categories.remove(category);
	}

	/**
	 * @param ID
	 *            category ID for the search in Data base
	 * @return category the category with the especific ID null if category dos
	 *         not exists
	 * @throws NullObjectException
	 */
	public Category searchCategory(Integer ID) throws NullObjectException {


		Category category = this.categories.search(ID);
		return category;
	}

	/**
	 * @return Collection the collection of all existing languages
	 */
	public Collection searchAllCategories() {


		Collection categories = this.categories.searchAll();
		return categories;
	}

	/**
	 * @return Collection the collection of all existing languages with the
	 *         specific language
	 */
	public Collection searchCategoriesByLanguage(Integer languageID) {


		Collection categories = new ArrayList();
		try {
			categories = this.categories.searchByLanguage(languageID);
		} catch (NullObjectException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return categories;
	}

	/*
	 * ***************************CATEGORYSERVICE*****************************
	 */

	/**
	 * Stores a categoryService object in the DB
	 * 
	 * @param categoryService
	 *            categoryService to be stored
	 * @throws NullObjectException
	 * @throws ExistingObjectException
	 */
	public void registerCategoryService(CategoryService categoryService)
	throws ExistingObjectException, NullObjectException {
		this.categoryServices.register(categoryService);
	}

	/**
	 * Returns a categoryService object searched by its identification in the
	 * DB.
	 * <p>
	 * If the categoryService isn't found it returns null.
	 * 
	 * @param categoryID,
	 *            serviceID the identification in the DB
	 * @return a categoryService object
	 * @throws NullObjectException
	 */
	public CategoryService searchCategoryService(Integer categoryID,
			Integer ID_Service) throws NullObjectException {


		CategoryService categoryService = this.categoryServices.search(categoryID, ID_Service);
		return categoryService;
	}

	/**
	 * Returns a categoryService collection object searched by the service
	 * identification in the DB.
	 * <p>
	 * If the categoryService isn't found it returns null.
	 * 
	 * @param ID_Service
	 *            the identification in the DB
	 * @return a categoryService collection object
	 */
	public Collection searchCategoryServiceByService(Integer ID_Service) {


		Collection categoryServices = new ArrayList();
		try {
			categoryServices = this.categoryServices.searchByService(ID_Service);
		} catch (NullObjectException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return categoryServices;
	}

	/**
	 * Removes a categoryService object from the DB
	 * 
	 * @param categoryService
	 *            categoryService to be removed
	 * @throws NullObjectException
	 * @throws NonExistingObjectException
	 */
	public void removeCategoryService(CategoryService categoryService)
	throws NonExistingObjectException, NullObjectException {
		this.categoryServices.remove(categoryService);
	}

	/**
	 * Returns a collection of all categoryService object stored in the DB.
	 * <p>
	 * If any categoryService isn't found it returns an empty collection.
	 * 
	 * @return a categoryService collection
	 */
	public Collection searchAllCategoryServices() {


		Collection categoryServices = this.categoryServices.searchAll();
		return categoryServices;
	}

	/*
	 * ***************************CATEGORYPLACE*****************************
	 */

	/**
	 * Stores a categoryPlace object in the DB
	 * 
	 * @param categoryPlace
	 *            categoryPlace to be stored
	 * @throws NullObjectException
	 * @throws NonExistingObjectException
	 */
	public void registerCategoryPlace(CategoryPlace categoryPlace)
	throws NonExistingObjectException, NullObjectException {
		this.categoryPlaces.register(categoryPlace);
	}

	/**
	 * Returns a categoryPlace object searched by its identification in the DB.
	 * <p>
	 * If the categoryPlace isn't found it returns null.
	 * 
	 * @param categoryID,
	 *            placeID the identification in the DB
	 * @return a categoryPlace object
	 * @throws NullObjectException
	 */
	public CategoryPlace searchCategoryPlace(Integer categoryID, Integer placeID)
	throws NullObjectException {


		CategoryPlace categoryPlace = this.categoryPlaces.search(categoryID, placeID);
		return categoryPlace;
	}

	/**
	 * Removes a categoryPlace object from the DB
	 * 
	 * @param categoryPlace
	 *            categoryPlace to be removed
	 * @throws NullObjectException
	 * @throws NonExistingObjectException
	 */
	public void removeCategoryPlace(CategoryPlace categoryPlace)
	throws NonExistingObjectException, NullObjectException {
		this.categoryPlaces.remove(categoryPlace);
	}

	/**
	 * Returns a collection of all categoryPlace object stored in the DB.
	 * <p>
	 * If any categoryPlace isn't found it returns an empty collection.
	 * 
	 * @return a categoryPlace collection
	 */
	public Collection searchAllCategoryPlaces() {


		Collection categoryPlace = this.categoryPlaces.searchAll();
		return categoryPlace;
	}

	public Collection searchCategoryPlaceByPlace(Integer placeID)
	throws NullObjectException {


		Collection categoryPlace = this.categoryPlaces.searchByPlace(placeID);
		return categoryPlace;
	}

	public Collection searchCategoryPlaceByLanguage(Integer languageID)
	throws NullObjectException {


		Collection categoryPlace = this.categoryPlaces.searchByLanguage(languageID);
		return categoryPlace;
	}

	/*
	 * ***************************CATEGORYACTIVITY*****************************
	 */

	/**
	 * Stores a categoryActivity object in the DB
	 * 
	 * @param categoryActivity
	 *            categoryActivity to be stored
	 * @throws NullObjectException
	 * @throws ExistingObjectException
	 */
	public void registerCategoryActivity(CategoryActivity categoryActivity)
	throws ExistingObjectException, NullObjectException {
		this.categoryActivities.register(categoryActivity);
	}

	/**
	 * Returns a categoryActivity object searched by its identification in the
	 * DB.
	 * <p>
	 * If the categoryActivity isn't found it returns null.
	 * 
	 * @param categoryID,
	 *            activityID the identification in the DB
	 * @return a categoryActivity object
	 * @throws NullObjectException
	 */
	public CategoryActivity searchCategoryActivity(Integer categoryID,
			Integer activityID) throws NullObjectException {


		CategoryActivity categoryActivity = this.categoryActivities.search(categoryID, activityID);
		return categoryActivity;
	}

	/**
	 * Removes a categoryActivity object from the DB
	 * 
	 * @param categoryActivity
	 *            categoryActivity to be removed
	 * @throws NullObjectException
	 * @throws NonExistingObjectException
	 */
	public void removeCategoryActivity(CategoryActivity categoryActivity)
	throws NonExistingObjectException, NullObjectException {
		this.categoryActivities.remove(categoryActivity);
	}

	/**
	 * Returns a collection of all categoryActivity object stored in the DB.
	 * <p>
	 * If any categoryActivity isn't found it returns an empty collection.
	 * 
	 * @return a categoryActivity collection
	 */
	public Collection searchAllCategoryActivities() {
		Collection categoryActivity = this.categoryActivities.searchAll();
		return categoryActivity;
	}

	/**
	 * Returns a collection of all categoryActivity object stored in the DB with
	 * the specific language.
	 * <p>
	 * If any categoryActivity isn't found it returns an empty collection.
	 * 
	 * @param languageID
	 * @return a categoryActivity collection
	 * @throws NullObjectException
	 */
	public Collection searchCategoryActivitiesByLanguage(Integer languageID)
	throws NullObjectException {


		Collection categoryActivity = categoryActivity = this.categoryActivities
		.searchByLanguage(languageID);
		return categoryActivity;

	}

	/**
	 * 
	 * @param activityID
	 * @return
	 */
	public Collection searchCategoryActivityByActivity(Integer activityID) {


		Collection categoryActivity = new ArrayList();
		try {
			categoryActivity = this.categoryActivities
			.searchByActivity(activityID);
		} catch (NullObjectException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return categoryActivity;
	}

	/*
	 * *****************PLACEAREA**************************************
	 */

	/**
	 * 
	 */
	public void registerPlaceArea(PlaceArea placeArea)
	throws ExistingObjectException, NullObjectException {
		this.placeAreas.register(placeArea);

	}

	/**
	 * 
	 * @param placeArea
	 * @throws NonExistingObjectException
	 * @throws NullObjectException
	 */
	public void removePlaceArea(PlaceArea placeArea)
	throws NonExistingObjectException, NullObjectException {
		this.placeAreas.remove(placeArea);
	}

	/**
	 * 
	 * @param id
	 * @return
	 * @throws NullObjectException
	 */
	public PlaceArea searchPlaceArea(Integer id) throws NullObjectException{


		PlaceArea placeArea = this.placeAreas.search(id);
		return placeArea;
	}

	/**
	 * 
	 * @param placeID
	 * @param areaID
	 * @return
	 * @throws NullObjectException
	 */
	public PlaceArea searchPlaceAreaByPlaceArea(Integer placeID, Integer areaID)
	throws NullObjectException {



		PlaceArea placeArea = this.placeAreas.searchByPlaceArea(placeID, areaID);

		return placeArea;
	}

	/**
	 * 
	 * @param placeArea
	 * @throws NonExistingObjectException
	 * @throws NullObjectException
	 */
	public void modifyPlaceArea(PlaceArea placeArea)
	throws NonExistingObjectException, NullObjectException {
		this.placeAreas.modify(placeArea);
	}

	/**
	 * 
	 * @return
	 */
	public Collection searchAllPlaceAreas() {


		Collection placesArea = this.placeAreas.searchAll();

		return placesArea;
	}


	/**
	 * 
	 * @param placeID
	 * @return
	 * @throws NullObjectException
	 */
	public Collection searchPlaceAreasByPlace(Integer placeID)
	throws NullObjectException {


		Collection placesArea = this.placeAreas.searchByPlace(placeID);
		return placesArea;
	}


	/**
	 * 
	 * @param areaID
	 * @param languageID
	 * @return
	 * @throws NullObjectException
	 */
	public Collection searchPlaceAreasByArea(Integer areaID, Integer languageID)
	throws NullObjectException {


		Collection placesArea = this.placeAreas.searchByArea(areaID, languageID);

		return placesArea;
	}

}
