package com.efetiva.wiiser.server.tests;

import java.io.IOException;
import java.util.Collection;
import java.util.Date;
import java.util.Iterator;

import junit.framework.TestCase;

import com.efetiva.wiiser.server.comm.ServerCommunication;
import com.efetiva.wiser.entity.model.ActivityModel;
import com.efetiva.wiser.entity.model.HeaderModel;
import com.efetiva.wiser.entity.model.LanguageModel;
import com.efetiva.wiser.entity.model.MapModel;
import com.efetiva.wiser.entity.model.PlaceModel;
import com.efetiva.wiser.entity.model.ServiceModel;
import com.efetiva.wiser.exception.NullObjectException;


public class ServerCommunicationTest extends TestCase {

	public void testRequestActivityInformation() throws NullObjectException {
		System.out.println("Running testRequestActivityInformation...");
		try {
			ServerCommunication server = ServerCommunication.getInstance();
			
			ActivityModel actM = null;
			actM = server.requestActivityInformation(new Integer(1));
			assertTrue(actM != null);
			assertEquals(actM.getHeaderModel().getId().intValue(), 1);
			
			actM = null;
			actM = server.requestActivityInformation(new Integer(1));
			
			
			actM = server.requestActivityInformation(new Integer(2));
			
		} catch (IOException e) {
			assertEquals(true, false);
			e.printStackTrace();
		}
		
		System.out.println("Finished\n");
		
	}

	public void testRequestDayActivities() {
		System.out.println("Running testRequestDayActivities...");
		try {
			ServerCommunication server = ServerCommunication.getInstance();
			
			Collection allHeaderAct = null;
			allHeaderAct = server.requestDayActivities(new Date(), new Integer(1));
			assertNotNull(allHeaderAct);
			Iterator it = allHeaderAct.iterator();
			HeaderModel headerModel = null;
			if (it.hasNext()) {
				headerModel = (HeaderModel)it.next();
			}
			assertNotNull(headerModel);
			assertEquals(headerModel.getId().intValue(), 1);
			
			allHeaderAct = server.requestDayActivities(new Date(), new Integer(2));
			assertNotNull(allHeaderAct);
			it = allHeaderAct.iterator();
			headerModel = null;
			if (it.hasNext()) {
				headerModel = (HeaderModel)it.next();
			}
			assertNull(headerModel);
			
		} catch (IOException e) {
			assertFalse(true);
			e.printStackTrace();
		}
		
		System.out.println("Finished\n");
	}

	public void testRequestAreaActivities() {
		System.out.println("Running testRequestAreaActivities...");
		try {
			ServerCommunication server = ServerCommunication.getInstance();
			
			Collection allHeaderModel = null;
			allHeaderModel = server.requestAreaActivities(new Integer(1), new Integer(1));
			assertNotNull(allHeaderModel);
			Iterator it = allHeaderModel.iterator();
			HeaderModel headerModel = null;
			if (it.hasNext()) {
				headerModel = (HeaderModel)it.next();
			}
			assertNotNull(headerModel);
			assertEquals(headerModel.getId().intValue(), 1);
			
			allHeaderModel = server.requestAreaActivities(new Integer(1), new Integer(2));
			assertNotNull(allHeaderModel);
			it = allHeaderModel.iterator();
			headerModel = null;
			if (it.hasNext()) {
				headerModel = (HeaderModel)it.next();
			}
			assertNull(headerModel);
			
			allHeaderModel = server.requestAreaActivities(new Integer(2), new Integer(1));
			assertNotNull(allHeaderModel);
			it = allHeaderModel.iterator();
			headerModel = null;
			if (it.hasNext()) {
				headerModel = (HeaderModel)it.next();
			}
			assertNull(headerModel);
			
		} catch (IOException e) {
			assertFalse(true);
			e.printStackTrace();
		}
		
		System.out.println("Finished\n");
	}

	public void testRequestLanguages() {
		System.out.println("Running testRequestLanguages...");
		try {
			ServerCommunication server = ServerCommunication.getInstance();
			
			Collection languages = null;
			languages = server.requestLanguages();
			assertNotNull(languages);
			Iterator it = languages.iterator();
			LanguageModel languageModel = null;
			int i = 1;
			while (it.hasNext()) {
				languageModel = (LanguageModel)it.next();
				if (i==1) {
					assertEquals(languageModel.getName(), "portugues");
				} else {
					assertEquals(languageModel.getName(), "ingles");
				}
				i++;
			}
			assertNotNull(languageModel);
			
		} catch (IOException e) {
			assertFalse(true);
			e.printStackTrace();
		}
		
		System.out.println("Finished\n");
	}

	public void testRequestMap() {
		System.out.println("Running testRequestMap...");
		try {
			ServerCommunication server = ServerCommunication.getInstance();
			
			MapModel mapM = null;
			mapM = server.requestMap(new Integer(1));
			assertTrue(!mapM.getAreasCoordInfo().isEmpty());
			
			mapM = null;
			mapM = server.requestMap(new Integer(2));
			assertTrue(mapM.getAreasCoordInfo().isEmpty());
			
			mapM = null;
			mapM = server.requestMap(new Integer(3));
			assertTrue(mapM.getAreasCoordInfo().isEmpty());
			
		} catch (IOException e) {
			assertFalse(true);
			e.printStackTrace();
		}
		
		System.out.println("Finished\n");
	}

	public void testRequestAllServices() {
		System.out.println("Running testRequestAllServices...");
		try {
			ServerCommunication server = ServerCommunication.getInstance();
			
			Collection allServices = null;
			allServices = server.requestAllServices(new Integer(1));
			assertTrue(!allServices.isEmpty());
			
			allServices = server.requestAllServices(new Integer(2));
			assertTrue(allServices.isEmpty());
			
			allServices = server.requestAllServices(new Integer(3));
			assertTrue(allServices.isEmpty());
			
		} catch (IOException e) {
			assertFalse(true);
			e.printStackTrace();
		}
		
		System.out.println("Finished\n");
		
	}

	public void testRequestAreaServices() {
		System.out.println("Running testRequestAreaServices...");
		try {
			ServerCommunication server = ServerCommunication.getInstance();
			
			Collection allServices = null;
			allServices = server.requestAreaServices(new Integer(1), new Integer(1));
			assertTrue(!allServices.isEmpty());
			
			allServices = server.requestAreaServices(new Integer(1), new Integer(2));
			assertTrue(allServices.isEmpty());
			
			allServices = server.requestAreaServices(new Integer(1), new Integer(3));
			assertTrue(allServices.isEmpty());
			
			allServices = server.requestAreaServices(new Integer(2), new Integer(1));
			assertTrue(allServices.isEmpty());
			
			allServices = server.requestAreaServices(new Integer(2), new Integer(2));
			assertTrue(allServices.isEmpty());
			
		} catch (IOException e) {
			assertFalse(true);
			e.printStackTrace();
		}
		
		System.out.println("Finished\n");
	}

	public void testRequestServiceInformation() {
		System.out.println("Running testRequestServiceInformation...");
		try {
			ServerCommunication server = ServerCommunication.getInstance();
			
			ServiceModel service = null;
			service = server.requestServiceInformation(new Integer(1));
			assertNotNull(service);
			
			service = null;
			service = server.requestServiceInformation(new Integer(1));
			assertNull(service);
			
			service = null;
			service = server.requestServiceInformation(new Integer(2));
			assertNull(service);
			
		} catch (IOException e) {
			fail(e.getMessage());
			e.printStackTrace();
		}
		
		System.out.println("Finished\n");
	}

	public void testRequestAllPlaces() {
		System.out.println("Running testRequestAllPlaces...");
		try {
			ServerCommunication server = ServerCommunication.getInstance();
			
			Collection allPlaces = null;
			allPlaces = server.requestAllPlaces(new Integer(1));
			assertTrue(!allPlaces.isEmpty());
			
			allPlaces = null;
			allPlaces = server.requestAllPlaces(new Integer(2));
			assertTrue(allPlaces.isEmpty());
			
			allPlaces = null;
			allPlaces = server.requestAllPlaces(new Integer(3));
			assertTrue(allPlaces.isEmpty());
			
		} catch (IOException e) {
			assertFalse(true);
			e.printStackTrace();
		}
		
		System.out.println("Finished\n");
	}

	public void testRequestAreaPlaces() {
		System.out.println("Running testRequestAreaPlaces...");
		try {
			ServerCommunication server = ServerCommunication.getInstance();
			
			Collection allPlaces = null;
			allPlaces = server.requestAreaPlaces(new Integer(1), new Integer(1));
			assertTrue(!allPlaces.isEmpty());
			
			allPlaces = server.requestAreaPlaces(new Integer(1), new Integer(2));
			assertTrue(allPlaces.isEmpty());
			
			allPlaces = server.requestAreaPlaces(new Integer(1), new Integer(3));
			assertTrue(allPlaces.isEmpty());
			
			allPlaces = server.requestAreaPlaces(new Integer(2), new Integer(1));
			assertTrue(allPlaces.isEmpty());
			
			allPlaces = server.requestAreaPlaces(new Integer(2), new Integer(2));
			assertTrue(allPlaces.isEmpty());
			
		} catch (IOException e) {
			assertFalse(true);
			e.printStackTrace();
		}
		
		System.out.println("Finished\n");
	}

	public void testRequestPlaceInformation() {
		System.out.println("Running testRequestplaceInformation...");
		try {
			ServerCommunication server = ServerCommunication.getInstance();
			
			PlaceModel place = null;
			place = server.requestPlaceInformation(new Integer(1), new Integer(1));
			assertNotNull(place);
			
			place = null;
			place = server.requestPlaceInformation(new Integer(1), new Integer(1));
			assertNull(place);
			
			place = null;
			place = server.requestPlaceInformation(new Integer(2), new Integer(1));
			assertNull(place);
			
		} catch (IOException e) {
			fail(e.getMessage());
			e.printStackTrace();
		}
		
		System.out.println("Finished\n");
	}

}
